/* plotaz.h ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s.j.schwartz@qmw.ac.uk)
*  Queen Mary, University of London
*  26 November 2000
*     May 2001 up-date after re-organization of Qpeace routines
*     Jan 2003 includes mapping, shifting, ...
*
*  New address:
*     s.schwartz@imperial.ac.uk
*     Blackett Laboratory, Imperial College London
*/

#ifndef PLOTAZ_H
#define PLOTAZ_H

#ifndef __cplusplus
#define __cplusplus
#endif


#include <qvbox.h>
#include <qhbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qtextbrowser.h>
#include <qpushbutton.h>
#include <qhbuttongroup.h>
#include <qstring.h>
#include <qlabel.h>
#include <qframe.h>
#include <qlineedit.h>
#include <qapplication.h>
#include <qdialog.h>
#include <qtooltip.h>
#include <qfiledialog.h>
#include <qdatetimeedit.h> 
#include <qspinbox.h> 

#include "Qsweep_spin.h"
#include "Qpeace_pad.h"
#include "Qidfs_utils.h"
#include "Qpeace_plot.h"

#define PLOTAZ_VERSION_STR "V3.6.1"
#define PLOTAZ_DEFAULT_XWIN_STR "53"
#define N_MAX_CUTS 6
#define PLOTAZ_LIOUVILLE_UNSHIFT_DOUBLE 7
#define PLOTAZ_LIOUVILLE_SHIFTED_DOUBLE 5
#define PLOTAZ_SHIFT_THETA_FAC 2
#define PLOTAZ_SHIFT_LEVEL_FAC 2

#define MAXLENGTH_SAVE_LINE  512

class QLabel;

static const char *project_strs[] = {"CLUSTERII", "DOUBLESTAR", "UNKNOWN", 0};
//static const char *mission_strs[] = {"C1", "C2", "C3", "C4", "TC-1", "TC-2",
//                   "FLIGHTSPARE", 0};  // done in Qidfs_utils.c 
static const char *exp_strs[] = {"PEACE", 0};
static const char *inst_strs[] = {"PAD", "SPINPAD", "3DR", "3DF", "LER", 
         "CORE", "NOI", "MODE", "3DX1", "3DX2", "HSKP", "RPAD", 0};
static const char *virins_strs[] = {
      "CPPADH" , "CPPADHF" , "CPPADL" , "CPPADLF",  "CPPADAL",// PAD
      "LSPAD"  , "HSPAD"   , "LSPADNR", "HSPADNR" ,           // SPINPAD
      "CP3DRH" , "CP3DRHF" , "CP3DRL" , "CP3DRLF",            // 3DR
      "CP3DFH" , "CP3DFHF" , "CP3DFL" , "CP3DFLF",            // 3DF
      "CPLERH" , "CPLERHF" , "CPLERL" , "CPLERLF",            // LER
      "CPX1L" , "CPX1H" , "CPXP1L" , "CPXP1H" , "CPXE1L" , "CPXE1H" , "CPX1LF",
         "CPX1HF" , "CPXP1LF" , "CPXP1HF" , "CPXE1LF" , "CPXE1HF",  // 3DX1
      "CPX2L" , "CPX2H" , "CPXP2L" , "CPXP2H" , "CPXE2L" , "CPXE2H" , "CPX2LF",
         "CPX2HF" , "CPXP2LF" , "CPXP2HF" , "CPXE2LF" , "CPXE2HF",  // 3DX2
       "DP3DF","DP3DR", "DPRPADL", "DPRPADH", 0};


#define N_POSSIBLE_CUT_SOURCES 11   /* includes NO_SOURCE */

typedef enum SWEEP_SOURCE_E{
    NO_SOURCE = 0,
    PRIMARY,
    SHIFTED_PRIMARY,
    MAPPED_PRIMARY,
    SHIFTED_THEN_MAPPED_PRIMARY,
    SECONDARY,
    SHIFTED_SECONDARY,
    MAPPED_OVER_SECONDARY,
    SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY,
    MAPPED_MINUS_SECONDARY,
    SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY
} SWEEP_SOURCE;
    
typedef enum PA_CHOICE_E{
    NO_BINNING = 0,
    PA_ANGLES,
    PA_COSINES
} PA_CHOICE;


typedef enum PLOT_CHOICE_E{
    LGIF,
    PGIF,
    LPS,
    PPS,
    WHEEL_ANIM,
    XWIN
} PLOT_CHOICE;     /* nb XWIN last, others in order of pull down */   

typedef enum ACTIONS_E{
    XDEV,
    PRINT2FILE,
    XWIN_N_SLICE,
    HELP,
    QUIT
} ACTIONS;




class LineageBox : public QVBox
{
    Q_OBJECT

public:
    LineageBox (QWidget *parent = 0, const char *name = 0);
    QComboBox *proj, *miss, *exp, *inst, *virins;

protected:
};

class TimeBox : public QHBox
{
    Q_OBJECT
public:
    TimeBox(QWidget *parent=0, const char *name =0);
    
    QLabel* yearLabel;
    QLabel* TimeLabel;
    QLabel* timeLabel;
    QDateEdit* dateEdit;
    QTimeEdit* timeEdit;
    QLabel* doyLabel;
    QLabel* textLabel1_2;
    QSpinBox* DoyspinBox;
    QSpinBox* YearspinBox;

   void GetTimeFromTimeBox(IDFS_TIME *);
   
   void setSHour(int);
   void setSMinute(int);
   void setSSecond(int);
   
public slots:
    void DateChange();
    void YearChange();
    void DoyChange();
};

class SweepBox : public QGroupBox
{
    Q_OBJECT

public:
    SweepBox (int strips = 1, Orientation=Horizontal,
                    QWidget *parent = 0, const char *name = 0);
    QLabel *project, *mission, *exp, *inst, *virins;
    QComboBox *pachoice;
    QLineEdit *npa;
    QComboBox *gatherchoice;
    QLineEdit *gathern;
    QComboBox *sc_pot;
    QLineEdit *sc_pot_value, *sc_pot_gap;
//    QLineEdit *year, *doy, *hr, *min, *sec; 
    QLineEdit *azslice;
    QComboBox *onecountchoice;
    QLineEdit *onecount_mult;
    TimeBox  *timebox;
    IDFS_TIME etime; /* hold end time here */
    void SetLineage(LineageBox *);
    void transfer_lineage( IDFS_LINEAGE *lng);
    void transfer_sweep_times( IDFS_TIME *idfta);

protected:
    QLabel *lineage_lbl, *time_lbl, *time_lbl2, *slice_lab;
    // more to go here to set time and az slice

protected slots:
     // void slotCopyStart2EndTime(void);
     void slotForceGather(int);
     void slotNextSegment(void);
};



class Sources : public QComboBox
{
   Q_OBJECT
   
public:
     Sources (QWidget *parent = 0, const char *name =0);

protected:
protected slots:

};

class Pangles : public QLineEdit
{
   Q_OBJECT
   
public:
   Pangles (QWidget *parent = 0, const char *name =0);

protected:
protected slots:

};

class Linestys : public QComboBox
{
   Q_OBJECT
   
public:
     Linestys (QWidget *parent = 0, const char *name =0);

protected:
protected slots:

};


class Colours : public QComboBox
{
   Q_OBJECT
   
public:
     Colours (QWidget *parent = 0, const char *name =0);

protected:
protected slots:

};

class Linesthick : public QComboBox
{
   Q_OBJECT
   
public:
     Linesthick (QWidget *parent = 0, const char *name =0);

protected:
protected slots:

};

class Cuts_Selector : public QVBox
{
   Q_OBJECT
   
public:
   QLabel  *srclab;
   Sources *source;
   Pangles *pangle;
   Linestys *linesty;
   Colours  *colour;
   Linesthick *thick;     
   Cuts_Selector (int icut, QWidget *parent = 0, const char *name = 0);
};

class Helpbwin : public QVBox
{
    Q_OBJECT

public:
      Helpbwin (QWidget *parent = 0, const char *name = 0);
      ~Helpbwin();

protected:

protected slots:

};

class Slicewin : public QDialog
{
    Q_OBJECT

public:

    QLineEdit *b1, *b2, *pot, *v1, *v2;
    Sources *lwheel, *rwheel;
    QComboBox *box_thick;
    Cuts_Selector *cuts[N_MAX_CUTS];
    QComboBox  *energy_units, *energy_linlog, *energy_scale;
    QComboBox  *psd_units, *psd_linlog, *psd_scale, *plot_col, *plot_dev;
    QLineEdit  *emin, *emax, *psdmin, *psdmax, *plot_win_edit, *plot_file_edit;
    QComboBox *sc_pot;
    QLineEdit *sc_pot_value, *sc_pot_gap;

    Slicewin (QWidget *parent =0, const char *name = 0);
    ~Slicewin();

protected:

protected slots:
      void slotDismiss(void);
      void slotRun(int);
      void slotPlotfile(void);

private:
      float bprim, bsec, potential,vprim, vsec;
      float thetas[N_MAX_CUTS];
      SWEEP_SOURCE lwhsrc, rwhsrc, cutsrc[N_MAX_CUTS];
      QPGLine_colour cutcol[N_MAX_CUTS];
      QPGLinestyle cutlsty[N_MAX_CUTS];
      float cutthick[N_MAX_CUTS];
      PEACE_SWEEP *swpp[N_MAX_CUTS];
      PEACE_SWEEP *swposs[N_POSSIBLE_CUT_SOURCES];
      int nswps, nwheels, swpcount[N_POSSIBLE_CUT_SOURCES];
      SWEEP_CUT_OPTIONS sweep_cut_options, *scop;
      SWEEP_WHEEL_OPTIONS sweep_wheel_options, *swop; 
      PGSpectro_ret pg_ret_val;

};

class PlotAzBox : public QVBox
{
    Q_OBJECT

public:
    PlotAzBox( QWidget *parent = 0, const char *name = 0 );
    LineageBox *lineage;
    SweepBox *prim_sweep, *sec_sweep;
    QComboBox  *energy_units, *energy_linlog, *energy_scale;
    QComboBox  *psd_units, *psd_linlog, *psd_scale, *plot_col, *plot_dev;
    QLineEdit  *emin, *emax, *psdmin, *psdmax, *plot_win_edit, *plot_file_edit;
    QComboBox  *anal_proc, *anal_linlog, *anal_scale;
    QLineEdit  *analmin, *analmax;
    QPushButton *quit;
    PEACE_SWEEP *swp1, *swp2,*swpar;
    PEACE_PITCH_ANGLE_OPTS paopts1, paopts2;
    IDFS_LINEAGE lng1, lng2;
    IDFS_TIME idft1a, idft1b, idft2a, idft2b;
    BIN_UNITS bin_units;
    PSD_UNITS psdu, psdu1, psdu2;
    PGSpectro_ret pg_ret_val;
    SWEEP_WHEEL_OPTIONS swoptions;
    float az1, az2, onecnt_mult1, onecnt_mult2;
    Helpbwin *helpbwin;
    Slicewin *slicewin;


protected:
//    LineageBox *lineage;
//    SweepBox *prim_sweep, *sec_sweep;


protected slots:
    void slotSetPrimarySweepLineage(void);
    void slotSetSecondarySweepLineage(void);
    void slotCopyTime(void);
    void slotActions(int);
    void slotPlotfile(void);
    void slotSlice(void);
    void slotHelp(void);
    void slotSave(void);
    void slotRestore(void);

private:

};


static const char *savestrings[] = {
   "PROJECT","MISSION","EXPERIMENT", "INSTRUMENT","VIRTUAL",
   "PROJECT1","MISSION1","EXPERIMENT1", "INSTRUMENT1","VIRTUAL1",
   "PA_OPTS1","SN_PA1",
   "GATHER_OPTS1","N_GATHER1",
   "SP_POT_OPTS1", "SC_POT_VAL1", "SC_POT_SECS1",
   "YEAR1","DOY1","HR1","MIN1","SEC1",
   "AZ1","ONECNTCH1","ONECNT_MULT1",
   "PROJECT2","MISSION2","EXPERIMENT2", "INSTRUMENT2","VIRTUAL2",
   "PA_OPTS2","SN_PA2",
   "GATHER_OPTS2","N_GATHER2",
   "SP_POT_OPTS2", "SC_POT_VAL2", "SC_POT_SECS2",
   "YEAR2","DOY2","HR2","MIN2","SEC2",
   "AZ2","ONECNTCH2","ONECNT_MULT2",
   "ENERGY_UNITS","ENERGY_LINLOG","ENERGY_AUTOSCALE","EMIN","EMAX",
   "PSD_UNITS", "PSD_LINLOG","PSD_SCALE","PSD_MIN","PSD_MAX",   
   "ANALYSIS_OPT","ANALYSIS_LINLOG","ANALYSIS_AUTOSCALE",
   "ANALYSIS_MIN", "ANALYSIS_MAX",
   "PLOT_COL","PLOT_DEV","XWIN_NO","PLOT_STEM",
   /* slicewin */
   "VPARA1","VPARA2","B1","B2","DELPHI",
   "LEFT_WHEEL","RIGHT_WHEEL","BOX_THICK",
   "CUT_OPT0","CUT_ANG0","CUT_STYLE0","CUT_COL0","CUT_THICK0",
   "CUT_OPT1","CUT_ANG1","CUT_STYLE1","CUT_COL1","CUT_THICK1",
   "CUT_OPT2","CUT_ANG2","CUT_STYLE2","CUT_COL2","CUT_THICK2",
   "CUT_OPT3","CUT_ANG3","CUT_STYLE3","CUT_COL3","CUT_THICK3",
   "CUT_OPT4","CUT_ANG4","CUT_STYLE4","CUT_COL4","CUT_THICK4",
   "CUT_OPT5","CUT_ANG5","CUT_STYLE5","CUT_COL5","CUT_THICK5",
   "SLICE_E_LINLOG","SLICE_E_AUTO","SLICE_EMIN","SLICE_EMAX",
   "SLICE_PSD_LINLOG","SLICE_PSD_AUTO","SLICE_PSDMIN","SLICE_PSDMAX",
   "SLICE_PLOT_COL","SLICE_PLOT_DEV","SLICE_XWIN_NO","SLICE_PLOT_STEM",
   "DUMMY_STRING"
};


typedef enum SAVE_STR_E{   
   SPROJECT,SMISSION,SEXPERIMENT, SINSTRUMENT,SVIRTUAL,
   SPROJECT1,SMISSION1,SEXPERIMENT1, SINSTRUMENT1,SVIRTUAL1,
   SPA_OPTS1,SN_PA1,
   SGATHER_OPTS1,SN_GATHER1,
   SSC_POT_OPTS1, SSC_POT_VAL1, SSC_POT_SECS1,
   SYR1, SDOY1, SHR1, SMIN1, SSEC1,
   SAZ1,SONECNTCH1,SONECNT_MULT1,
   SPROJECT2,SMISSION2,SEXPERIMENT2, SINSTRUMENT2,SVIRTUAL2,
   SPA_OPTS2,SN_PA2,
   SGATHER_OPTS2,SN_GATHER2,
   SSC_POT_OPTS2, SSC_POT_VAL2, SSC_POT_SECS2,
   SYR2, SDOY2, SHR2, SMIN2, SSEC2,
   SAZ2,SONECNTCH2,SONECNT_MULT2,
   SENERGY_UNITS,SENERGY_LINLOG,SENERGY_AUTOSCALE,SEMIN, SEMAX,
   SPSD_UNITS, SPSD_LINLOG, SPSD_SCALE, SPSD_MIN, SPSD_MAX,
   SANALYSIS_OPT,SANALYSIS_LINLOG,SANALYSIS_AUTOSCALE,
   SANALYSIS_MIN, SANALYSIS_MAX,
   SPLOT_COL, SPLOT_DEV, SXWIN_NO, SPLOT_STEM,  
   /* slicewin */
   SVPARA1,SVPARA2,SB1,SB2,SDELPHI,
   SLEFT_WHEEL,SRIGHT_WHEEL,SBOX_THICK,
   SCUT_OPT0,SCUT_ANG0,SCUT_STYLE0,SCUT_COL0,SCUT_THICK0,
   SCUT_OPT1,SCUT_ANG1,SCUT_STYLE1,SCUT_COL1,SCUT_THICK1,
   SCUT_OPT2,SCUT_ANG2,SCUT_STYLE2,SCUT_COL2,SCUT_THICK2,
   SCUT_OPT3,SCUT_ANG3,SCUT_STYLE3,SCUT_COL3,SCUT_THICK3,
   SCUT_OPT4,SCUT_ANG4,SCUT_STYLE4,SCUT_COL4,SCUT_THICK4,
   SCUT_OPT5,SCUT_ANG5,SCUT_STYLE5,SCUT_COL5,SCUT_THICK5,
   SSLICE_E_LINLOG,SSLICE_E_AUTO,SSLICE_EMIN, SSLICE_EMAX,
   SSLICE_PSD_LINLOG,SSLICE_PSD_AUTO,SSLICE_PSDMIN, SSLICE_PSDMAX,
   SSLICE_PLOT_COL, SSLICE_PLOT_DEV, SSLICE_XWIN_NO, SSLICE_PLOT_STEM, 
   SDUMMYSTRING    
}SAVE_STR;




#endif
