/* qjas.h ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s.j.schwartz@qmw.ac.uk)
*  Queen Mary, University of London
*   June 2001  hacked from plotaz.h
*
*  Now: s.schwartz@imperial.ac.uk
*  Blackett Laboratory, Imperial College London
*  2005- 
*
*/

#ifndef QJAS_H
#define QJAS_H

#ifndef __cplusplus
#define __cplusplus
#endif

#include <qvbox.h>
#include <qhbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qtextbrowser.h>
#include <qpushbutton.h>
#include <qhbuttongroup.h>
#include <qstring.h>
#include <qlabel.h>
#include <qframe.h>
#include <qlineedit.h>
#include <qapplication.h>
#include <qtooltip.h>
#include <qsizepolicy.h>
#include <qfiledialog.h>
#include <qdatetimeedit.h> 
#include <qspinbox.h> 

#include "Qsweep_spin.h"
#include "Qpeace_pad.h"
#include "Qidfs_utils.h"
#include "Qpeace_plot.h"

#define QJAS_VERSION_STR "V3.6.1a"  /* version a fixes sc_potential timetags on export */
#define PLOTAZ_DEFAULT_XWIN_STR "53"

#define MAXLENGTH_SAVE_LINE  512

class QLabel;

static const char *project_strs[] = {"CLUSTERII", "DOUBLESTAR", "UNKNOWN", 0};
//static const char *mission_strs[] = {"C1", "C2", "C3", "C4", "TC-1", "TC-2",
//                   "FLIGHTSPARE", 0};  // done in Qidfs_utils.c 
static const char *exp_strs[] = {"PEACE", 0};
static const char *inst_strs[] = {"PAD", "SPINPAD", "3DR", "3DF", "LER", 
         "CORE", "NOI", "MODE", "3DX1", "3DX2", "HSKP", "RPAD", 0};
static const char *virins_strs[] = {
      "CPPADH" , "CPPADHF" , "CPPADL" , "CPPADLF", "CPPADAL", // PAD
      "LSPAD"  , "HSPAD"   , "LSPADNR", "HSPADNR" ,           // SPINPAD
      "CP3DRH" , "CP3DRHF" , "CP3DRL" , "CP3DRLF",            // 3DR
      "CP3DFH" , "CP3DFHF" , "CP3DFL" , "CP3DFLF",            // 3DF
      "CPLERH" , "CPLERHF" , "CPLERL" , "CPLERLF",            // LER
      "CPX1L" , "CPX1H" , "CPXP1L" , "CPXP1H" , "CPXE1L" , "CPXE1H" , "CPX1LF",
         "CPX1HF" , "CPXP1LF" , "CPXP1HF" , "CPXE1LF" , "CPXE1HF",  // 3DX1
      "CPX2L" , "CPX2H" , "CPXP2L" , "CPXP2H" , "CPXE2L" , "CPXE2H" , "CPX2LF",
         "CPX2HF" , "CPXP2LF" , "CPXP2HF" , "CPXE2LF" , "CPXE2HF",  // 3DX2
       "DP3DF","DP3DR", "DPRPADL", "DPRPADH", 0};


typedef enum PA_CHOICE_E{
    NO_BINNING = 0,
    PA_ANGLES,
    PA_COSINES
} PA_CHOICE;

typedef enum DEV_CHOICE_E{
    LGIF,
    PGIF,
    LPS,
    PPS,
    XWIN
} DEV_CHOICE;     /* nb XWIN last, others in order of pull down */   

typedef enum PLOT_CHOICE_E{
    QJAS,
    WHEEL_ANIM,
    WHEEL_CUTS_ANIM
} PLOT_CHOICE;

class LineageBox : public QVBox
{
    Q_OBJECT

public:
    LineageBox (QWidget *parent = 0, const char *name = 0);
    QComboBox *proj, *miss, *exp, *inst, *virins;
    QComboBox *pachoice;
    QLineEdit *npa;
    QComboBox *gatherchoice;
    QLineEdit *gathern;
    void grab_lineage(IDFS_LINEAGE *lngp);

protected:

};


/* class TimeBox : public QGroupBox
{
    Q_OBJECT

public:
    TimeBox (int strips = 1, Orientation=Horizontal,
                    QWidget *parent = 0, const char *name = 0);
    QLineEdit *year, *doy, *hr, *min, *sec;
    void GetTimeFromTimeBox(IDFS_TIME *);

protected:
    QLabel  *time_lbl;

protected slots:

};
*/

class TimeBox : public QGroupBox
{
    Q_OBJECT
public:
    TimeBox(QWidget *parent=0, const char *name =0);
    
    QLabel* byearLabel;
    QLabel* StartTimeLabel;
    QLabel* btimeLabel;
    QDateEdit* StartdateEdit;
    QPushButton* CopyTimepushButton;
    QDateEdit* EnddateEdit;
    QTimeEdit* etimeEdit;
    QLabel* etimeLabel;
    QTimeEdit* btimeEdit;
    QLabel* edoyLabel;
    QLabel* bdoyLabel;
    QLabel* textLabel1_2;
    QLabel* textLabel1_2_2;
    QLabel* eyearLabel;
    QSpinBox* eDoyspinBox;
    QSpinBox* bDoyspinBox;
    QSpinBox* bYearspinBox;
    QSpinBox* eYearspinBox;
    QLabel* EndTimetextLabel;

   void GetStartTimeFromTimeBox(IDFS_TIME *);
   void GetEndTimeFromTimeBox(IDFS_TIME *);
   
   void setSHour(int);
   void setSMinute(int);
   void setSSecond(int);
   void setEHour(int);
   void setEMinute(int);
   void setESecond(int);
   
public slots:
    void StartDateChange();
    void EndDateChange();
    void StartYearChange();
    void EndYearChange();
    void StartDoyChange();
    void EndDoyChange();
    void CopyTime();
};


class Helpbwin : public QVBox
{
    Q_OBJECT
    
public:
      Helpbwin (QWidget *parent = 0, const char *name = 0);
      ~Helpbwin();

protected:

protected slots:

};


class QjasBox : public QVBox
{
    Q_OBJECT

public:
    QjasBox( QWidget *parent = 0, const char *name = 0 );
    LineageBox *lineage;
    TimeBox    *startendbox;
    QComboBox  *plottype;
    QComboBox  *energy_units, *energy_bins, *energy_sc_pot;
    QComboBox  *energy_linlog, *energy_scale;
    QComboBox  *psd_units, *psd_linlog, *psd_scale, *plot_col, *plot_dev;
    QLineEdit  *sc_pot_gap_secs, *sc_pot_val;
    QLineEdit  *emin, *emax, *psdmin, *psdmax, *plot_win_edit, *plot_file_edit;
    QLineEdit  *nebins,*plot_tgap;
    QComboBox  *exp_opt;
    QLineEdit  *exp_file_edit;
    QPushButton *quit;
    PEACE_SWEEP **swppp;   /* array of ptrs to sweep structures; array malloced */
    PEACE_PITCH_ANGLE_OPTS paopts;
    PEACE_SWEEP **cong_swps;
    int nswp;
    IDFS_LINEAGE lng1;
    IDFS_TIME idft1, idft2;
    BIN_UNITS bin_units;
    PSD_UNITS psdu;
    PGSpectro_ret pg_ret_val;
    PGSpectro pgspectro;
    Helpbwin *helpbwin;

protected:

protected slots:
//    void slotCopyTime(void);
    void slotExp(void);
    void slotExpfile(void);
    void slotPlotfile(void);
    void slotPlot(int);
    void slotHelp(void);
    void slotSave(void);
    void slotRestore(void);

private:

};

static const char *savestrings[] = {
   "PROJECT","MISSION","EXPERIMENT", "INSTRUMENT","VIRTUAL",
   "PA_OPTS","N_PA",
   "GATHER_OPTS","N_GATHER",
   "START_YR", "START_DOY", "START_HR", "START_MIN", "START_SEC",
   "END_YR", "END_DOY", "END_HR", "END_MIN", "END_SEC", "PLOT_TYPE",
   "ENERGY_UNITS", "ENERGY_BINS","SP_POT_OPTS", "SC_POT_VAL", "SC_POT_SECS",
   "CUSTOM_BIN_NO", "CUTSOM_BIN_LINLOG","CUSTOM_BIN_RANGE","CUSTOM_BIN_MIN",
   "CUSTOM_BIN_MAX",
   "PSD_UNITS", "PSD_LINLOG","PSD_SCALE","PSD_MIN","PSD_MAX",
   "PLOT_COL","PLOT_DEV","XWIN_NO","PLOT_STEM","PLOT_TGAP",
   "EXPORT_FORMAT","EXPORT_STEM","DUMMY_STRING"
};

typedef enum SAVE_STR_E{   
   SPROJECT,SMISSION,SEXPERIMENT, SINSTRUMENT,SVIRTUAL,
   SPA_OPTS,SN_PA,
   SGATHER_OPTS,SN_GATHER,
   SSTART_YR, SSTART_DOY, SSTART_HR, SSTART_MIN, SSTART_SEC,
   SEND_YR, SEND_DOY, SEND_HR, SEND_MIN, SEND_SEC, SPLOT_TYPE,
   SENERGY_UNITS, SENERGY_BINS, SSC_POT_OPTS, SSC_POT_VAL, SSC_POT_SECS,
   SCUSTOM_BIN_NO, SCUSTOM_BIN_LINLOG, SCUSTOM_BIN_RANGE, SCUSTOM_BIN_MIN,
   SCUSTOM_BIN_MAX,
   SPSD_UNITS, SPSD_LINLOG, SPSD_SCALE, SPSD_MIN, SPSD_MAX,
   SPLOT_COL, SPLOT_DEV, SXWIN_NO, SPLOT_STEM, SPLOT_TGAP,
   SEXPORT_FORMAT, SEXPORT_STEM,
   SDUMMYSTRING
}SAVE_STR;


#endif
