/* Main programme for Qjas_plot development
*
*  sjs
*  24 April 2001
*  8 May    2001  try out PAD sweep gathering routines
*
*/

#include "Qsweep_spin.h"
#include "Qpeace_pad.h"
#include "Qidfs_utils.h"
#include "Qpeace_plot.h"
 


int main(){

/* INPUTS************************************************************/
IDFS_LINEAGE lineage = { "CLUSTERII",     /* project */
                         "CLUSTER-2",     /* mission */
                         "PEACE",         /* experiment */
                         "PAD",           /* instrument */
                         "CPPADL",        /* vir instr */
			 "",              /* extension */
                         -100             /* dummy version number */
                       };

int syear  = 2001;    int eyear  = 2001;
int sdoy   =   90;    int edoy   =   90;
int shour  =    17;    int ehour  =   17;
int smin   =   0;    int emin   =   5;
int ssec   =    00;    int esec   =    0;
long snano =    0;    long enano =    0;

BIN_UNITS bin_units = BIN_UNITS_EV;    /* BIN_UNITS_KM_S or BIN_UNITS_EV; */
PSD_UNITS psd_units = PSD_UNITS_DIFF_ENERGY_FLUX; 
/* PSD_UNITS_CNTS_ACCUM; */ /* or PSD_UNITS_S3_PER_KM6; */
/**********************************************************************/

IDFS_TIME btime,etime;
SDDAS_SHORT ret_val;

IDFS_LINEAGE *lngp;

PGSpectro_ret pgsret;
PEACE_SWEEP **swppp;   /* array of ptrs to sweep structures; array malloced */
PEACE_SWEEP **cong_swps;
int i, npads;

PGSpectro pgspectro, *pgsp;

pgsp=&pgspectro;
lngp=&lineage; 

for(;;){   /* forever loop to test memory */


to_idfs_time( syear, sdoy, shour, smin, ssec, snano, &btime);
to_idfs_time( eyear, edoy, ehour, emin, esec, enano, &etime);

printf("btime = ");print_idfs_time(&btime);
printf("\nlasttime = ");print_idfs_time(&etime);printf("\n");

ret_val = GetNPeaceSweeps(lngp,&btime,&etime,bin_units,psd_units, QPTRUE, &npads, &swppp);
if(ret_val != ALL_OKAY){
   printf("GetNPeaceSweeps failed returning %d\n",ret_val);
   exit(-1);
}

/* if (cpgopen("?Device ?") <=0){ */
if (cpgopen("/xser") <=0){
   printf("Failed to open pgplot device successfully\n");
   exit(0);
}
cpgshls(0,0,1,0);
cpgshls(1,0,0,0);
 
PGSpectro_set_defaults(pgsp);
sweeps_autoscaler(swppp, npads, &(pgsp->zscalemin), &(pgsp->zscalemax)); 

/* pgsp->zscalemax = 4.e-4; */

cong_swps=NULL;

pgsp->zlinlog = PGSPECTRO_LOG;
pgsret=Qjas_plot(swppp, npads, pgsp, btime, etime);
if (pgsret == PGSPECTRO_QJAS_INCONGRUENT){
   cong_swps = (PEACE_SWEEP **)malloc(npads * sizeof (PEACE_SWEEP *));
   for (i=0; i<npads; i++){
      cong_swps[i] = rebin_sweep(swppp[0], swppp[i]);
   }
   pgsret=Qjas_plot(cong_swps, npads, pgsp, btime, etime);
}
 
cpgend();

free_n_sweeps(npads,swppp);
free(swppp);
swppp=NULL;

if (cong_swps != NULL){
  free_n_sweeps(npads,cong_swps);
  free(cong_swps);
  cong_swps = NULL;
}

printf("hit return");
getc(stdin);

}  /* end infinite loop */


exit(0);

}
