/* Main programme for sweep export development
*
*  sjs
*  July 2001 based on QSAS prog example*
*/

#include  "Qpeace_plot.h"
#include  "Qpeace_pad.h"    /* includes Qsweep_spin.h, which includes qie.h */

void QiCDFContentsDump(QiCDFContents *QiCDF);

int main(){

/* INPUTS************************************************************/
IDFS_LINEAGE lineage = { "CLUSTERII",     /* project */
                         "CLUSTER-2",     /* mission */
                         "PEACE",         /* experiment */
                         "3DR",           /* instrument */
                         "CP3DRL",        /* vir instr */
			 "",              /* extension */
                         -100             /* dummy version number */
                       };

int syear  =  2003;    int eyear  =  2003;
int sdoy   =    95;    int edoy   =    95;
int shour  =    22;    int ehour  =    22;
int smin   =     0;    int emin   =    0;
int ssec   =     0;    int esec   =    10;
long snano =     0;    long enano =     0;

BIN_UNITS bin_units = BIN_UNITS_EV;    /* BIN_UNITS_KM_S or BIN_UNITS_EV; */
PSD_UNITS psd_units =  PSD_UNITS_CNTS_ACCUM; /* PSD_UNITS_DIFF_ENERGY_FLUX*/ 
/* PSD_UNITS_CNTS_ACCUM; */ /* or PSD_UNITS_S3_PER_KM6; */
/**********************************************************************/

IDFS_TIME btime,etime;
SDDAS_SHORT ret_val;

IDFS_LINEAGE *lngp;

PGSpectro_ret pgsret;
PEACE_SWEEP **swppp;   /* array of ptrs to sweep structures; array malloced */
PEACE_SWEEP **cong_swps;
int i, nswps;

PEACE_SC_POTENTIAL phisc;

PGSpectro pgspectro, *pgsp;

pgsp=&pgspectro;
lngp=&lineage; 


cong_swps = NULL;

to_idfs_time( syear, sdoy, shour, smin, ssec, snano, &btime);
to_idfs_time( eyear, edoy, ehour, emin, esec, enano, &etime);

printf("btime = ");print_idfs_time(&btime);
printf("\nlasttime = ");print_idfs_time(&etime);printf("\n");

for(;;){   /* forever loop to test memory */


ret_val = GetNPeaceSweeps(lngp,&btime,&etime,bin_units,psd_units,QPTRUE,&nswps, &swppp);
if(ret_val != ALL_OKAY){
   printf("GetNPeaceSweeps failed returning %d\n",ret_val);
   exit(-1);
}

SC_Pot_init(&phisc);  /* includes setting most of lineage */
strcpy(phisc.phisc_lineage.mission,lngp->mission); /* copy Cluster s/c */

ret_val = SC_Pot_fetch(&phisc,&btime,&etime);
switch(ret_val){
   case ALL_OKAY: printf(" ALL_OKAY\n"); break;
   case PHI_SC_EFW_NONFATAL_ERROR: printf("Non fatal error\n"); break;
   case PHI_SC_FATAL_ERROR: printf("Fatal error\n"); break;
   case PHI_SC_EOF: printf("Eof but ok\n"); break;
   default: printf("Unknown return value = %d\n",ret_val);
}

if (ret_val == PHI_SC_FATAL_ERROR) continue;  

ret_val = phisc2sweep_all(&phisc,swppp,nswps);
/* setting DIAG_PHISC will dump all results */

ret_val = sweep_sc_potential_adjust(swppp,nswps);
if (ret_val != ALL_OKAY){
   printf("Oops, sweep_sc_potential failed\n");
   continue;
}

if (cpgopen("?Device ?") <=0){ 
   printf("Failed to open pgplot device successfully\n");
   exit(0);
}
cpgshls(0,0,1,0); cpgshls(1,0,0,0);
 
PGSpectro_set_defaults(pgsp);
sweeps_autoscaler(swppp, nswps, &(pgsp->zscalemin), &(pgsp->zscalemax)); 

cong_swps=NULL;

pgsp->zlinlog = PGSPECTRO_LOG;
pgsret=Qjas_plot(swppp, nswps, pgsp, btime, etime);
if (pgsret == PGSPECTRO_QJAS_INCONGRUENT){
   cong_swps = (PEACE_SWEEP **)malloc(nswps * sizeof (PEACE_SWEEP *));
   for (i=0; i<nswps; i++){
      cong_swps[i] = rebin_sweep(swppp[0], swppp[i]);
   }
   pgsret=Qjas_plot(cong_swps, nswps, pgsp, btime, etime);
}
 cpgend();


free_n_sweeps(nswps,swppp);
free(swppp);
swppp=NULL;

if (cong_swps != NULL){
  free_n_sweeps(nswps,cong_swps);
  free(cong_swps);
  cong_swps = NULL;
}


printf("  hit return");
getc(stdin);

}  /* end infinite loop */


exit(0);

}

