/* Main programme for plotaz_repeater
*
*  sjs
*  Jan 2007
*
*  Perform plotaz plots on a sequence in batch-like mode. Use
*  plotaz gui to set everything up and save to a plotaz save file,
*  then pass that file and time interval in secs as command-line 
*  argument to plot a sequence of wheels.
*
*  1st version makes no attempt to watch out for gaps or otherwise
*  harmonise primary and secondary sweeps.
*
*  1st version doesn't do any azimuthal selection, nor anything on the
*  1D Cuts/Liouville mapping... gui, nor any sweep arithmetic.
*/

#include  "Qpeace_pad.h"    /* includes everything else */

#define RESTORE_FILE_ERROR -900

void print_usage();
void read_plotaz_saveset(char *savefile,
  PEACE_LINEAGE *lngp, 
  PA_CHOICE *pach, 
  int *npa, 
  PEACE_GATHER_CHOICE *gatherch,
  float *gathersecs,
  int *gather_swp_spins,
  PHISC_CHOICE *phiscch,
  float *sc_pot_val,
  float *sc_pot_gap,
  IDFS_TIME *idft,
  int *one_cnt_choice,
  float *one_cnt_mult,
  BIN_UNITS  *bin_unitsp,
  PSD_UNITS  *psd_unitsp,
  SWEEP_WHEEL_OPTIONS  *swp_wheel_optsp
)

int main(int argc , char* argv[]){

/* command line inputs are 
  argv[1] plotaz saveset
  argv[2] time interval in secs 
  (argv[0] is name of programme as called on command line)*/

/* separate primary and secondary sweep quantities */
PEACE_GATHER_GAPS    gapopts[2];
PEACE_GATHER_CHOICE  gatherch[2];
float                gathersecs[2];
int                  gather_swp_spins[2];
PEACE_PITCH_ANGLE_OPTS paopts[2];
PHISC_CHOICE         phiscch[2];
float                sc_pot_val[2], sc_pot_gap[2];
PEACE_LINEAGE        lngp[2];
PEACE_SWEEP          **swpp[2];
IDFS_TIME            idft[2];
int                  one_cnt_choice[2];
float                one_cnt_mult[2];

/* common settings */
BIN_UNITS            bin_units;
PSD_UNITS            psd_units;
SWEEP_WHEEL_OPTIONS  swp_wheel_opts;

float                time_interval_secs;

if(argc != 3){
  print_usage();
  exit(0);
}

/* go fill inputs */
time_interval_secs=argv[2];
read_plotaz_saveset(argv[1],
  lngp, 
  paopts, 
  gatherch,
  gathersecs,
  gather_swp_spins,
  phiscch,
  sc_pot_val,
  sc_pot_gap,
  idft,
  one_cnt_choice,
  one_cnt_mult,
  &bin_units,
  &psd_units,
  &swp_wheel_optsp
); /* 1st sets defaults for everything */

if (cpgopen("? Device ?") <=0){
 printf("Failed to open pgplot device  successfully\n");
 printf("If file check write permissions; if Xwindow, check pgxwin server\n");
 return;       // early exit, before getting sweeps and malloc memory
}
cpgshls(0,0,1,0);
cpgshls(1,0,0,0);

/* is_pad = strcmp("PAD",lng1.instrument);
if (is_pad==0) is_pad=1; else is_pad = 0; */

for (isc = 0; isc < nsc; isc++){
    cpgpage();
    PGSpectro_domain_set_defaults(&(pgspectro.domain));
    pgspectro.wedge.disp = 2.5;  /* leave room for Flow 0 180 label */
    if (nsc == 4) {
       pgspectro.char_size = 1.4;
       strcpy(lng1.mission, SC_NAME((SC_E)isc));
    }

    int phisc_ch = energy_sc_pot->currentItem();
    switch ( phisc_ch ){
       case NO_SC_POT_CORR:
          ret_val = GetNPeaceSweeps(&lng1,&idft1,&idft2,bin_units,psdu,
               paopts.do_pa, &nswp, &swppp);
          break;
       case DO_SC_POT_CORR :  /* not const s/c pot */
          ret_val = GetNPotCorrectedPeaceSweeps(
	        &lng1,&idft1,&idft2,bin_units,psdu,paopts.do_pa, 
		QPFALSE, sc_pot_val->text().toFloat(),
		sc_pot_gap_secs->text().toFloat(),
		&nswp, &swppp);
	  break;
       case CONST_SC_POT :
          ret_val = GetNPotCorrectedPeaceSweeps(
	        &lng1,&idft1,&idft2,bin_units,psdu,paopts.do_pa, 
		QPTRUE, sc_pot_val->text().toFloat(),
		sc_pot_gap_secs->text().toFloat(),
		&nswp, &swppp);
	  break;
       default:
          printf("Unknown SC Potential Choice");
	  ret_val = PEACE3D_ERROR;
	  break;
    }
    if (ret_val != ALL_OKAY){
       printf("QJAS failed initial Get Sweeps returning %d.",ret_val);
       printf(" Try another source/time?\n");
       break;
    }        
       
/* set up bins for rebinning, but don't do it yet */
/* Note pa-rebinning requires congruent sweeps: same levs and sensors */
/* Note rebin_n_sweeps does NOT copy pitch angles ! */
/* So one approach is:
*
*  Rebin each sweep (one by one) into pitch angles - at initial angle resol.
*  Then Gather/combine onto final bin spec:
*
*/

    if (paopts.do_pa == QPTRUE){   /* 1st rebin each sweep to pa; old bins */
       single_paopts = paopts;
       single_paopts.n_pabins = swppp[0]->n_sen;
       single_paopts.mu_bins  = QPFALSE;    /* keep as close as poss to orig*/
       
       ret_val = Rebin_n_sweeps_to_pa(swppp, nswp, 1, &single_paopts,
                 &swppp_pa,&nswp_pa);
       free_n_sweeps(nswp,swppp); free(swppp); swppp=NULL; 
                                          /*  free orig ptr*/ 
       if (ret_val != ALL_OKAY){
          printf("Qjas failed to rebin each sweep in pa, ret = %d\n",ret_val);
	  break;	  
       }
       nswp = nswp_pa;
       swppp=swppp_pa;   swppp_pa = NULL;
    }

    binh = create_sweep_full_copy(swppp[0]); /* use existing unless:*/
    if (energy_bins->currentItem() == 1){ // custom bins requested
	 free(binh->elo);
	 free(binh->ehi);
	 nlev = binh->n_lev = nebins->text().toInt(); // set up ebins
	 if ( energy_scale->currentItem() == 1){            // user scale
	     emin_val = emin->text().toFloat();
	     emax_val = emax->text().toFloat();
	 } else {  /* be careful as could have negative bins */
	     i=0;
	     while (i<swppp[0]->n_lev) {
	       emin_val=swppp[0]->elo[i++];
	       if (emin_val > 0.0) break;
	     }  
	     if (emin_val < 0) emin_val = SAFE_EMIN;
	     emax_val = swppp[0]->ehi[swppp[0]->n_lev -1];
	     if (emax_val < 0) emax_val = SAFE_EMAX;
	 }
	 binh->elo = (float *)malloc(nlev*sizeof(float));
	 binh->ehi = (float *)malloc(nlev*sizeof(float));
	 set_equal_bins_linlog(binh->elo, binh->ehi, nlev, 
	            emin_val, emax_val, 
		    (PGSpectro_linlog)energy_linlog->currentItem());
    }
    if (paopts.do_pa == QPTRUE){
	free(binh->thlo); free(binh->thhi);
	nsen = binh->n_sen = paopts.n_pabins;
	lobins = binh->thlo = (float *)malloc(nsen*sizeof(float));
	hibins = binh->thhi = (float *)malloc(nsen*sizeof(float));
	if (paopts.mu_bins == QPTRUE){
	   set_equal_bins( lobins, hibins, nsen, 1.0, -1.0);
	   for (i=0; i<nsen; i++){
	      lobins[i] = RAD2DEG*acos( lobins[i]);
	      hibins[i] = RAD2DEG*acos( hibins[i]);
	   }
	   lobins[0] = 0.0;  /* force to elim rounding error */
	   hibins[nsen -1] = 180.0;
	} else {
	   set_equal_bins (lobins, hibins, nsen, 0.0, 180.0);
	}
    }

    sweep_spin_gap_defaults (&gapopts);  /* default swp, spin gap */
    if (is_pad) gapopts.sweep_gap = gapopts.spin_gap;
    
    gatherch= (PEACE_GATHER_CHOICE)lineage->gatherchoice->currentItem();
    gathersecs = lineage->gathern->text().toFloat();
    gather_swp_spins = lineage->gathern->text().toInt();
    
    ret_val = gather_sweeps(swppp, nswp, binh, gatherch, gapopts, gathersecs,
       gather_swp_spins, &gath_swppp, &ngath);
    Set_sweep_wheel_options_default(&swopts);
    swopts.rmin = binh->elo[0];
    swopts.rmax = binh->ehi[binh->n_lev -1];
    free_sweep_arrays(binh); free(binh);
    if (ret_val != ALL_OKAY){
       printf("gather sweeps failed returning %d\n",ret_val);
       free_n_sweeps(nswp,swppp); free(swppp);   swppp=NULL;
       break ;
    }
    free_n_sweeps(nswp,swppp); free(swppp); 
    swppp = gath_swppp; nswp = ngath; gath_swppp = NULL;  /* shuffle */  
    
    if(zautoscale == YES && isc == 0){
      sweeps_autoscaler(swppp, nswp, 
                    &(pgspectro.zscalemin), &(pgspectro.zscalemax)); 
    }
    pgspectro.zmissing = swppp[0]->missing_value; 

    if (animation == QPTRUE){
      swopts.color_table  = pgspectro.color_table;
      swopts.zlinlog      = pgspectro.zlinlog;
      swopts.rlinlog      = (PGSpectro_linlog)energy_linlog->currentItem();

      swopts.zautoscale   = zautoscale;
      swopts.rautoscale   = NO;
      swopts.zmin         = pgspectro.zscalemin;
      swopts.zmax         = pgspectro.zscalemax;
      swopts.skip_missing = YES;
      
       for (int i=0; i<nswp; i++){
           pg_ret_val = PlotTwoSweeps(swppp[i],swppp[i],&swopts);
           cpgpage();      
       }    
       emin->setText(s.setNum(swppp[0]->elo[0]));  /* set ebins in gui */
       emax->setText(s.setNum(swppp[0]->ehi[ swppp[0]->n_lev -1 ]));
//       free_n_sweeps(nswp,cong_swps);  free(cong_swps);    cong_swps=NULL;

    } else { /* qjas plot was here */
    }       



    free_n_sweeps(nswp,swppp);       free(swppp);        swppp=NULL;
}
cpgclos();   /* make sure pgplot dev is properly closed */

/* get the sweeps; QPFALSE turns off pitch_angle gathering */
ret_val = GetNPeaceSweeps(lngp,&btime,&etime,bin_units,psd_units, 
                       QPFALSE, &nswps, &swppp);
if(ret_val != ALL_OKAY){
   printf("GetNPeaceSweeps failed returning %d\n",ret_val);
   exit(-1);
}

/
free_n_sweeps(nswps,swppp);
free(swppp);
swppp=NULL;

exit(0);

}

void print_usage(){

  printf("Usage\n");
  printf("plotaz_repeater <plotaz saveset file>  <time interval in secs>");
return;

}

SDDAS_SHORT read_plotaz_saveset(char *savefile,
  PEACE_LINEAGE *lngp, 
  PEACE_PITCH_ANGLE_OPTS *paopts, 
  PEACE_GATHER_CHOICE *gatherch,
  float *gathersecs,
  int *gather_swp_spins,
  PHISC_CHOICE *phiscch,
  float *sc_pot_val,
  float *sc_pot_gap,
  IDFS_TIME *idft,
  int *one_cnt_choice,
  float *one_cnt_mult,
  BIN_UNITS  *bin_unitsp,
  PSD_UNITS  *psd_unitsp,
  SWEEP_WHEEL_OPTIONS  *swp_wheel_optsp
){

char inputline[MAXLENGTH_SAVE_LINE];

int i;
FILE *in;
int yr[2],doy[2],hr[2],mm[2],ss[2];

/* set everything to some safe default */

strcpy(lngp[0].project,"CLUSTERII");
strcpy(lngp[0].mission,"CLUSTER-2");
strcpy(lngp[0].experiment,"PEACE");
strcpy(lngp[0].instrument,"PAD");
strcpy(lngp[0].virtual_inst,"CPPADL");
strcpy(lngp[0].exten,"");
lngp[0].vsn=0;
copy_lineage(lngp[1],lngp[0]);
set_pa_opts_defaults(paopts[0]);
set_pa_opts_defaults(paopts[1]);
gatherch[0]=gatherch[1]=NO_GATHER;
gathersecs[0]=gathersecs[1]=0.0;
gather_swp_spins[0]=gather_swp_spins[1]=1;
phiscch[0] = phiscch[1]=NO_SC_POT_CORR;
sc_pot_val[0]=sc_pot_val[1]=0.0;
sc_pot_gap[0]=sc_pot_gap[1]=0.0;
yr[0]=yr[1]=2001;
doy[0]=doy[1]=90;
hh[0]=hh[1]=0;
mm[0]=mm[1]=0;
ss[0]=ss[1]=0;
one_cnt_choice[0]=one_cnt_choice[1]=0;
one_cnt_mult[0]=one_cnt_choice[1]=1.0;
*bin_units=BIN_UNITS_EV;
*psd_units=PSD_UNITS_CNTS_ACCUM;
Set_sweep_wheel_defaults(swp_wheel_optsp);

in = fopen(savefile,"r");
if (in == NULL){
   printf("Error opening %s for restore\n",savefile);
   return RESTORE_FILE_ERROR;
}

QString qin, qparam, qvalue;

while (fgets (inputline, MAXLENGTH_SAVE_LINE, in) != NULL){
    
    qin    = inputline;
    qparam = (qin.section("=",0,0)).stripWhiteSpace();
    qvalue = (qin.section("=",-1,-1)).stripWhiteSpace();

    // printf("qparam = |%s| and qvalue = |%s|\n",qparam.latin1(),qvalue.latin1());

    for (i=0; i<SDUMMYSTRING; i++){
        if(qparam ==savestrings[i])break;
    }
    if (i< SDUMMYSTRING){
       switch(i){

	 case SPROJECT1:      strcpy(lngp[0].project, project_strs(qvalue.toInt()));       break;
	 case SMISSION1:      strcpy(lnpg[0].mission,SC_NAME(qvalue.toInt()));             break;
	 case SEXPERIMENT1:   strcpy(lnpg[0].experiment,exp_strs(qvalue.toInt()));         break;
	 case SINSTRUMENT1:   strcpy(lnpg[0].instrument,inst_strs(qvalue.toInt()));        break;
	 case SVIRTUAL1:      strcpy(lnpg[0].virtual_inst,virins_strs(qvalue.toInt()));    break;
	 case SPA_OPTS1:      if (qvalue.toInt == NO_BINNING) paopts[0].do_pa=QPFALSE;
                              else {
                                 paopts[0].do_pa=QPTRUE;
                                 if (qvalue.toInt()== PA_ANGLES) paopts[0].mu_bins=QPFALSE;
                                 else  paopts[0].mu_bins=QPTRUE;
                              }                                                            break;
	 case SN_PA1:         paopts[0].n_pabins=qvalue.toInt();                           break;
	 case SGATHER_OPTS1:  gatherch[0]=qvalue.toInt();                                  break;
	 case SN_GATHER1:     gathersecs[0]=qvalue.toFloat();                              break;
	 case SSC_POT_OPTS1:  phiscch[0]=qvalue.toInt();                                   break;
	 case SSC_POT_VAL1:   sc_pot_val[0]=qvalue.toFloat();                              break;
	 case SSC_POT_SECS1:  sc_pot_gap[0]=qvalue.toFloat();                              break;
	 case SYR1:           yr[0]=qvalue.toInt();                                        break;
	 case SDOY1:          doy[0]=qvalue.toInt();                                       break;
	 case SHR1:           hr[0]=qvalue.toInt();                                        break;
	 case SMIN1:          mm[0]=qvalue.toInt();                                        break;
	 case SSEC1:          ss[0]=qvalue.toInt();                                        break;
	 case SONECNTCH1:     one_cnt_choice[0]=qvalue.toInt();                            break;
	 case SONECNT_MULT1:  one_cnt_mult[0]=qvalue.toFloat();                            break;

	 case SPROJECT2:      strcpy(lngp[1].project, project_strs(qvalue.toInt()));       break;
	 case SMISSION2:      strcpy(lnpg[1].mission,SC_NAME(qvalue.toInt()));             break;
	 case SEXPERIMENT2:   strcpy(lnpg[1].experiment,exp_strs(qvalue.toInt()));         break;
	 case SINSTRUMENT2:   strcpy(lnpg[1].instrument,inst_strs(qvalue.toInt()));        break;
	 case SVIRTUAL2:      strcpy(lnpg[1].virtual_inst,virins_strs(qvalue.toInt()));    break;
	 case SPA_OPTS2:      if (qvalue.toInt == NO_BINNING) paopts[1].do_pa=QPFALSE;
                              else {
                                 paopts[1].do_pa=QPTRUE;
                                 if (qvalue.toInt()== PA_ANGLES) paopts[1].mu_bins=QPFALSE;
                                 else  paopts[1].mu_bins=QPTRUE;
                              }                                                            break;
	 case SN_PA2:         paopts[1].n_pabins=qvalue.toInt();                           break;
	 case SGATHER_OPTS2:  gatherch[1]=qvalue.toInt();                                  break;
	 case SN_GATHER2:     gathersecs[1]=qvalue.toFloat();                              break;
	 case SSC_POT_OPTS2:  phiscch[1]=qvalue.toInt();                                   break;
	 case SSC_POT_VAL2:   sc_pot_val[1]=qvalue.toFloat();                              break;
	 case SSC_POT_SECS2:  sc_pot_gap[1]=qvalue.toFloat();                              break;
	 case SYR2:           yr[1]=qvalue.toInt();                                        break;
	 case SDOY2:          doy[1]=qvalue.toInt();                                       break;
	 case SHR2:           hr[1]=qvalue.toInt();                                        break;
	 case SMIN2:          mm[1]=qvalue.toInt();                                        break;
	 case SSEC2:          ss[1]=qvalue.toInt();                                        break;
	 case SONECNTCH2:     one_cnt_choice[1]=qvalue.toInt();                            break;
	 case SONECNT_MULT2:  one_cnt_mult[1]=qvalue.toFloat();                            break;

	 case SENERGY_UNITS:     *bin_units=qvalue.toInt();                                break;
	 case SENERGY_LINLOG:    swp_wheel_optsp->rlinlog=qvalue.toInt();                  break;
	 case SENERGY_AUTOSCALE: swp_wheel_optsp->rautoscale=qvalue.toInt();               break;
	 case SEMIN:          swp_wheel_optsp->rmin=qvalue.toFloat();                      break;
	 case SEMAX:          swp_wheel_optsp->rmax=qvalue.toFloat();                      break;
	 case SPSD_UNITS:     *psd_units->qvalue.toInt();                                  break;
	 case SPSD_LINLOG:    swp_wheel_optsp->zlinlog=qvalue.toInt();                     break; 
	 case SPSD_SCALE:     swp_wheel_optsp->zautoscale=qvalue.toInt();                  break;
	 case SPSD_MIN:       swp_wheel_optsp->zmin=qvalue.toFloat();                      break;
	 case SPSD_MAX:       swp_wheel_optsp->zmax=qvalue.toFloat();                      break;
	 default: break;
       }
    }

}

to_idfs_time( yr[0],doy[0],hh[0],mm[0],ss[0], 0, &(idft[0]));
to_idfs_time( yr[1],doy[1],hh[1],mm[1],ss[1], 0, &(idft[1]));

return ALL_OKAY;
}