/* Qsweep_export.c ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s.j.schwartz@qmw.ac.uk)
*  Queen Mary, University of London
*
*  May 2002 separate out from Qsweep_spin
*
*  Routines for exporting PEACE sweeps using qie modules
*
*/

#ifdef __cplusplus
extern "C"{
#endif

#include "Qidfs_utils.h"
#include "Qsweep_spin.h"
#include "Qpeace_pad.h"

/*********************************************************************/
SDDAS_SHORT nsweeps2QiCDFContents(PEACE_SWEEP **swppp, int nswps,
                                  QiCDFContents **QiCDFp){
/*********************************************************************/
/* take nswps sweeps and create and fill a QiCDFContents structure */

SDDAS_SHORT ret;
int i, dim, nth, nlev;

QiCDFVariable_novary novary_opts[10] = {EVERY_RECORD, EVERY_RECORD, EVERY_RECORD,
                EVERY_RECORD, EVERY_RECORD, 
		EVERY_RECORD, EVERY_RECORD,
		EVERY_RECORD, EVERY_RECORD,
		EVERY_RECORD};
long sizeofentrys[10] = {sizeof(double), sizeof(double), sizeof(float),
                        sizeof(float),  sizeof(float),  
			sizeof(float),  sizeof(float),  
			sizeof(float),  sizeof(float),
			sizeof(float)};
long numbers[10] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
char *names[10] = {"timetags",       "timetags_delta", "psd", /* 0, 1, 2 */
                  "phi",    "phi_delta",                   /* 3, 4    */
		  "theta",  "theta_delta",                 /* 5, 6    */
		  "level",  "level_delta",                 /* 7, 8    */
		  "sc_potential",};                        /* 9 */
long data_types[10] = {CDF_EPOCH, CDF_DOUBLE, CDF_FLOAT, CDF_FLOAT, CDF_FLOAT,  
		      CDF_FLOAT,  CDF_FLOAT, CDF_FLOAT, CDF_FLOAT, CDF_FLOAT};
long rec_varys[10]  = {VARY,    VARY,    VARY,    VARY,    VARY, 
                       VARY,    VARY,    VARY,    VARY,    VARY};
long num_dimss[10]   = {0, 0, 2, 0, 0, 1, 1, 1, 1, 0};
long num_elemss[10]  = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1};

long dim_size_epoch[1] = {0},       dim_size_epoch_delta[1] = {0}; 
long dim_size_psd[2];
long dim_size_phi[1] = {0},   dim_size_phi_delta[1] = {0};
long dim_size_theta[1], dim_size_theta_delta[1];
long dim_size_lev[1],   dim_size_lev_delta[1];
long dim_size_sc_pot[1]  = {0};       
long *dim_sizess[10] = {dim_size_epoch, dim_size_epoch_delta, dim_size_psd,
       dim_size_phi,   dim_size_phi_delta, 
       dim_size_theta, dim_size_theta_delta, 
       dim_size_lev,   dim_size_lev_delta,
       dim_size_sc_pot};
       
long dim_varys_epoch[1]       = {VARY}, dim_varys_epoch_delta[1] = {VARY};
long dim_varys_psd[2]         = {VARY, VARY}; 
long dim_varys_phi [1]        = {VARY},  dim_varys_phi_delta[1]   = {VARY};
long dim_varys_theta[1]       = {VARY},  dim_varys_theta_delta[1] = {VARY};
long dim_varys_lev[1]         = {VARY},  dim_varys_lev_delta[1]   = {VARY};
long dim_varys_sc_pot[1]      = {VARY};

long *dim_varyss[10] = {dim_varys_epoch, dim_varys_epoch_delta, dim_varys_psd,
      dim_varys_phi,   dim_varys_phi_delta,   
      dim_varys_theta, dim_varys_theta_delta,
      dim_varys_lev,   dim_varys_lev_delta,
      dim_varys_sc_pot};
      
QiCDFContents *QiCDF;

nth  = swppp[0]->n_sen;
nlev = swppp[0]->n_lev;
for (i=1; i<nswps; i++){   /* check sweeps all same size */
   if (swppp[i]->n_sen != nth || swppp[i]->n_lev != nlev){
      printf("nswps2QiCDF: Sweep %d has different length from 1st sweep.\n",i);
      return PEACE_SWEEPS_SIZE_DIFFER;
   }
}
/* set up as much as possible; details, incl attrs done in separate routines*/
dim_size_psd[0]         = nth;         dim_size_psd[1]         = nlev;
dim_size_theta[0]       = nth ;        dim_size_theta_delta[0] = nth;
dim_size_lev[0]         = nlev;        dim_size_lev_delta[0]   = nlev;

QiCDF = QiMakeCDFContentsObj();   /* ptrs all initialised to NULL or STR_NULL*/
*QiCDFp = QiCDF;                  /* pass ptr back to calling routine */
QiCDF->n_vars = 10;   /* epoch start, epoch_dplus,  psd, phi start, dplus,  */
                     /* theta start, dplus, levels start, dplus, sc pot     */
QiCDF->n_recs = nswps;
QiCDF->time_var_num = 0;
QiCDF->vardata = QiMakeQiVariablePtrs(QiCDF->n_vars);

ret= nswps2QiCDF_g_atts(QiCDF, swppp);

for (i=0; i<QiCDF->n_vars; i++){ 
    QiCDF->vardata[i] = QiMakeQiVariable();
    QiCDF->vardata[i]->novary_opt = novary_opts[i];
    QiCDF->vardata[i]->sizeofentry= sizeofentrys[i];
    QiCDF->vardata[i]->number     = numbers[i];
    QiCDF->vardata[i]->name       = QiNewStr(names[i]);
    QiCDF->vardata[i]->data_type  = data_types[i];
    QiCDF->vardata[i]->rec_vary   = rec_varys[i];
    QiCDF->vardata[i]->max_rec_num= nswps-1;
    QiCDF->vardata[i]->num_dims   = num_dimss[i];
    QiCDF->vardata[i]->num_elems  = num_elemss[i];
    for (dim = 0; dim < num_dimss[i]; dim++){  /* copy into malloced QiCDF */
	QiCDF->vardata[i]->dim_sizes[dim]  = *(dim_sizess[i]+dim);
	QiCDF->vardata[i]->dim_varys[dim]  = *(dim_varyss[i]+dim);
    }
}

ret = nswps2QiCDF_time_vars(QiCDF->vardata[0],QiCDF->vardata[1], swppp, nswps);
if (ret != ALL_OKAY) return ret;
ret = nswps2QiCDF_psd_var(QiCDF->vardata[2],swppp, nswps);
if (ret != ALL_OKAY) return ret;
ret = nswps2QiCDF_phi_vars(QiCDF->vardata[3], QiCDF->vardata[4], swppp, nswps);
if (ret != ALL_OKAY) return ret;
ret = nswps2QiCDF_theta_vars(QiCDF->vardata[5],QiCDF->vardata[6], swppp, nswps);
if (ret != ALL_OKAY) return ret;
ret = nswps2QiCDF_lev_vars(QiCDF->vardata[7], QiCDF->vardata[8], swppp, nswps);
if (ret != ALL_OKAY) return ret;
ret = nswps2QiCDF_phisc_var(QiCDF->vardata[9],swppp, nswps);
if (ret != ALL_OKAY) return ret;

for (i=0; i<QiCDF->n_vars; i++){
   if(QiVarSafe(QiCDF, i)==0){
      printf("QiCDF Variable %i failed safe test\n", i);
      return PEACE_SWEEPS_EXPORT_ERROR;
   }
}

return ret;

}

/*********************************************************************/
SDDAS_SHORT nswps2QiCDF_g_atts(QiCDFContents *QiCDF, PEACE_SWEEP **swppp){
/*********************************************************************/

char lineage_str[500],gen_time[33];
QiGlobalAttribute **g_att;
time_t now;
struct tm *date;

QiCDF->num_g_attrs = 3;   /* no mandatory globals; put lineage as global */
QiCDF->g_attr = QiMakeQiGAttrPtrs(QiCDF->num_g_attrs);

g_att = QiCDF->g_attr;  /* local copy */

g_att[0] = QiMakeQiGAttr();
g_att[0]->number = 0;
g_att[0]->name   = QiNewStr("IDFS Lineage");
g_att[0]->num_entries = 1;
g_att[0]->entry = QiMakeEntries(1);
g_att[0]->entry->exists = 1;    /*  ?????????????????????*/
g_att[0]->entry->data_type = CDF_CHAR;
sprintf(lineage_str,"%s %s %s %s",
       swppp[0]->lineage->mission,
       swppp[0]->lineage->experiment,
       swppp[0]->lineage->instrument,
       swppp[0]->lineage->virtual_inst);
g_att[0]->entry->data = QiNewStr(lineage_str);
g_att[0]->entry->num_elems = strlen(g_att[0]->entry->data) ;

g_att[1] = QiMakeQiGAttr();
g_att[1]->number = 1;
g_att[1]->name   = QiNewStr("TEXT");
g_att[1]->num_entries = 3;
g_att[1]->entry = QiMakeEntries(3);
g_att[1]->entry[0].exists = 1;   /*  yes, seems to need this*/
g_att[1]->entry[0].data_type = CDF_CHAR;
g_att[1]->entry[0].data = 
   QiNewStr("PEACE Sweep Data generated by QPEACE Software");
g_att[1]->entry[0].num_elems = strlen(g_att[1]->entry[0].data) ;
g_att[1]->entry[1].exists = 1;
g_att[1]->entry[1].data_type = CDF_CHAR;
g_att[1]->entry[1].data = 
   QiNewStr("s.schwartz@imperial.ac.uk; http://www.space-plasma.qmul.ac.uk");
g_att[1]->entry[1].num_elems = strlen(g_att[1]->entry[1].data);
g_att[1]->entry[2].exists = 1;   /*  yes, seems to need this*/
g_att[1]->entry[2].data_type = CDF_CHAR;
g_att[1]->entry[2].data = QiNewStr(qpeace_version_str());
g_att[1]->entry[2].num_elems = strlen(g_att[1]->entry[2].data) ;

g_att[2] = QiMakeQiGAttr();
g_att[2]->number = 2;
g_att[2]->name   = QiNewStr("Generation_date");
g_att[2]->num_entries = 1;
g_att[2]->entry = QiMakeEntries(1);
g_att[2]->entry[0].exists = 1;   /*  yes, seems to need this*/
g_att[2]->entry[0].data_type = CDF_CHAR;
now = time(NULL); date = localtime (&now);
strftime(gen_time,33,"%Y-%m-%dT%H:%M:%S.000Z Doy %j",date);
g_att[2]->entry[0].data = QiNewStr(gen_time);
g_att[2]->entry[0].num_elems = strlen(g_att[1]->entry[0].data) ;

return ALL_OKAY;
}

/*********************************************************************/
SDDAS_SHORT nswps2QiCDF_default_attr(QiVarAttribute *attp, int iatt){
/*********************************************************************/
attp->number = iatt;
attp->data_type = CDF_CHAR;
attp->num_elems = 1;   /* for V attrs (NOT g atts) this is no of entries */
return ALL_OKAY;
}

/*********************************************************************/
SDDAS_SHORT nswps2QiCDF_time_vars(QiCDFVariable *epo, QiCDFVariable *epo_delta,
                      PEACE_SWEEP **swppp, int nswps){
/**********************************************************************/
int i;
double *epodp, *epodpldp, t1,t2;
long num_tagatts = 6;
long num_datts = 4;

/* time attributes are Value_type (done by qie), UNITS, SI_conversion */
/* number, data_type set by nswps2QiCDF_default_attr */
/* NB qie routines set Time_format attribute 
*  NB QiMakeQiVariable allocates MAX_N_ATTRS attribute structures */

char *tagatt_names[6] = {"UNITS", "SI_CONVERSION","DELTA_PLUS","DELTA_MINUS" ,"PARAMETER_TYPE","VAR_TYPE"};
char *tagatt_data[6]  = {"msecs since 0AD", "1.e-3>s",
     "timetags_delta","timetags_delta","Support_Data","metadata"};
char *datt_names[4] = {"UNITS","SI_CONVERSION","PARAMETER_TYPE","VAR_TYPE"};
char *datt_data[4]  = {"sec","1.0>s","Support_Data","metadata"};

epo->num_v_attrs= num_tagatts;
epo_delta->num_v_attrs = num_datts;
for (i=0; i<num_tagatts; i++) {
  nswps2QiCDF_default_attr( &(epo->attribute[i]),i );
  epo->attribute[i].name = QiNewStr(tagatt_names[i]);
  epo->attribute[i].data = QiNewStr(tagatt_data[i]);
}
for (i=0; i<num_datts; i++){
  nswps2QiCDF_default_attr( &(epo_delta->attribute[i]),i );
  epo_delta->attribute[i].name  = QiNewStr(datt_names[i]);
  epo_delta->attribute[i].data  = QiNewStr(datt_data[i]);
}

epo->data     = (double *)malloc(nswps * sizeof(double));
epo_delta->data = (double *) malloc(nswps * sizeof(double));

epodp  = (double *)epo->data;
epodpldp = (double *)epo_delta->data;

for (i=0; i<nswps; i++){
    t1         = idfs_time_to_epoch_secs(&(swppp[i]->btime));
    t2         = idfs_time_to_epoch_secs(&(swppp[i]->etime));
    epodpldp[i]  = 0.5*(t2-t1);
    epodp[i]     = 500.0*(t1+t2);   /* epoch centre time in msecs */
}

return ALL_OKAY;
}

/*********************************************************************/
SDDAS_SHORT nswps2QiCDF_psd_var(QiCDFVariable *psdv, 
                                  PEACE_SWEEP **swppp, int nswps){
/**********************************************************************/
int i,j,k;
int n_per_swp;
float *psdvp;
float missing;
long num_atts = 10;  /* fillval added may2002; Depend_end_0 added Feb2003 */
                     /* SC_id and FIELDNAM added oct 2003  */
char fillvalstr[50];

/* array attributes for cef are Value_type,  FILLVAL, UNITS, SI_conversion,
*        Depend_1, Depend_end_1, Depend_2, Depend_end_2, 
*        Bin_description, Bin_description_A        
*        Sizes and Value_type added by qie routines */
/* number, data_type,  set by nswps2QiCDF_default_attr */

char *att_names[10] = {"FILLVAL","UNITS", "SI_CONVERSION","DEPEND_0",
       "DEPEND_1" , "DEPEND_2", 
       "Bin_description", "Bin_description_a","SC_ID","FIELDNAM"};
char *att_data[10]  = {"dummy fillval", "dummy units", "dummy SI_conv",
   "timetags","theta", "level",
   "Bins are polar theta vs speed/energy in an azimuthal slice",
   "Theta directions are instrument FLOW directions","dummy SC_id",
   "Phase Space Density"};

psdv->num_v_attrs= num_atts;
for (i=0; i<num_atts; i++) {
  nswps2QiCDF_default_attr( &(psdv->attribute[i]),i );
  psdv->attribute[i].name = QiNewStr(att_names[i]);
  psdv->attribute[i].data = QiNewStr(att_data[i]);
}
psdv->attribute[0].data_type = CDF_FLOAT;  /* fillval */

PQiChangeVattr("UNITS",PSD_UNITS_STRING(swppp[0]->psd_units),psdv);
PQiChangeVattr("SI_CONVERSION",
         PSD_UNITS_SI_conv_STRING(swppp[0]->psd_units),psdv);
sprintf(fillvalstr,"%g",swppp[0]->missing_value);
PQiChangeVattr("FILLVAL",fillvalstr,psdv);
PQiChangeVattr("SC_ID", swppp[0]->lineage->mission,psdv);

n_per_swp = swppp[0]->n_sen * swppp[0]->n_lev;
psdv->data     = (float *)malloc(nswps * n_per_swp * sizeof(float));

psdvp = (float *)psdv->data;

missing = swppp[0]->missing_value;
/* fill data array. NB psd is a 2-D array, but in right order so copy as 1-D*/
for (i=0; i<nswps; i++){
    k= i*n_per_swp;
    for (j=0; j<n_per_swp; j++){
        psdvp[k+j]  = swppp[i]->psd[j];
	if(!finitef(psdvp[k+j])) psdvp[k+j]=missing; /* remove NaNs */
    }
}

return ALL_OKAY;
}


/*********************************************************************/
SDDAS_SHORT nswps2QiCDF_phi_vars(QiCDFVariable *phiv, QiCDFVariable *phiv_delta,
                          PEACE_SWEEP **swppp, int nswps){
/**********************************************************************/
int iswp, i;
float *phivp, *phiv_dpldp, phi1, phi2;
long num_atts = 13;
long num_datts = 6;

/* Scalar variable attributes for cef are Value_type,  UNITS, SI_conversion,
*        Frame and Component_desc included for future use
*        FIELDNAM added for plots
*        Sizes and Value_type added by qie routines */
/* number, data_type  set by nswps2QiCDF_default_attr */


char *att_names[13] = {"TENSOR_RANK","UNITS", "SI_CONVERSION", "LABLAXIS",
       "FRAME", "REPRESENTATION", "COMPONENT", "FIELDNAM", 
       "DELTA_MINUS", "DELTA_PLUS", "TENSOR_FRAME", "PARAMTER_TYPE","VAR_TYPE"};
char *att_data[13]  = { "1","Deg", "1>degree", "PEA Az",
    "component>pea_rtp", "rtp", "p", "Peace tzero Az Flow Angle",
    "phi_delta","phi_delta", "pea","Support_Data","metadata"};

char *datt_names[6] = {"UNITS", "SI_CONVERSION", "LABLAXIS", "FIELDNAM","PARAMTER_TYPE","VAR_TYPE"};
char *datt_data[6]  = {"Deg", "1>degree","delta az", "Delta Pea Az","Support_Data","metadata"};

phiv->num_v_attrs= num_atts;
for (i=0; i<num_atts; i++) {
  nswps2QiCDF_default_attr( &(phiv->attribute[i]),i );
  phiv->attribute[i].name = QiNewStr(att_names[i]);
  phiv->attribute[i].data = QiNewStr(att_data[i]);
}
phiv->attribute[0].data_type=CDF_INT4;  /* rank */
PQiChangeVattr("TENSOR_RANK",att_data[0],phiv);

phiv_delta->num_v_attrs = num_datts;
for (i=0; i<num_datts; i++) {
  nswps2QiCDF_default_attr( &(phiv_delta->attribute[i]),i );
  phiv_delta->attribute[i].name = QiNewStr(datt_names[i]);
  phiv_delta->attribute[i].data = QiNewStr(datt_data[i]);
}

phiv->data         = (float *)malloc(nswps  * sizeof(float));
phivp              = (float *)phiv->data;
phiv_delta->data   = (float *)malloc(nswps  * sizeof(float));
phiv_dpldp         = (float *)phiv_delta->data;


for (iswp=0; iswp<nswps; iswp++){
    phi1             = swppp[iswp]->phi_lo;
    phi2             = swppp[iswp]->phi_hi;
    phivp[iswp]      = 0.5*(phi1+phi2);
    phiv_dpldp[iswp] = 0.5*(phi2-phi1);
}

return ALL_OKAY;
}

/*********************************************************************/
SDDAS_SHORT nswps2QiCDF_theta_vars(QiCDFVariable *thv, QiCDFVariable *thv_delta, 
               PEACE_SWEEP **swppp, int nswps){
/**********************************************************************/
int iswp, jth, k, nth,  i;
float *thvp, *thv_dpldp, th1, th2;
long num_atts = 13;
long num_datts = 6;

/* Dimension var attributes for cef are Value_type,  UNITS, SI_conversion,
*        Bin_location, Scaling; COULD ADD Frame & Component_desc 
*        Sizes and Value_type added by qie routines */
/* number, data_type, and num_elems set by nswps2QiCDF_default_attr */

char *att_names[13] = {"TENSOR_RANK","UNITS", "SI_CONVERSION", "LABLAXIS",
       "FRAME", "REPRESENTATION", "COMPONENT", "FIELDNAM", 
       "DELTA_MINUS", "DELTA_PLUS", "TENSOR_FRAME", "PARAMETER_TYPE","VAR_TYPE"};
char *att_data[13]  = {"1","Deg", "1>degree", "theta",
    "component>pea_rtp", "rtp", "t", "Peace Polar Flow Angle",
    "theta_delta","theta_delta", "pea", "Support_Data","metadata"};

char *datt_names[6] = {"UNITS", "SI_CONVERSION", "LABLAXIS", "FIELDNAM","PARAMETER_TYPE","VAR_TYPE"};
char *datt_data[6]  = {"Deg", "1>degree","delta theta", "Delta Pea theta","Support_Data","metadata"};


thv->num_v_attrs= num_atts;
for (i=0; i<num_atts; i++) {
  nswps2QiCDF_default_attr( &(thv->attribute[i]),i );
  thv->attribute[i].name = QiNewStr(att_names[i]);
  thv->attribute[i].data = QiNewStr(att_data[i]);
}
thv->attribute[0].data_type=CDF_INT4;  /* rank */
PQiChangeVattr("TENSOR_RANK",att_data[0],thv);

thv_delta->num_v_attrs = num_datts;
for (i=0; i<num_datts; i++) {
  nswps2QiCDF_default_attr( &(thv_delta->attribute[i]),i );
  thv_delta->attribute[i].name = QiNewStr(datt_names[i]);
  thv_delta->attribute[i].data = QiNewStr(datt_data[i]);
}


nth = swppp[0]->n_sen;

thv->data     = (float *)malloc(nswps * nth * sizeof(float));
thvp = (float *)thv->data;
thv_delta->data     = (float *)malloc(nswps * nth * sizeof(float));
thv_dpldp = (float *)thv_delta->data;

/* data is data[nswp][nth] so data[i][j] = data[i*nth + j] */

for (iswp=0; iswp<nswps; iswp++){
    k = iswp*nth;       /* indx of 1st elem in data for swpp[iswp] */
    for (jth = 0; jth < nth; jth++){
       th1             = swppp[iswp]->thlo[jth];
       th2             = swppp[iswp]->thhi[jth];
       thvp[k+jth]     = 0.5*(th1+th2);
       thv_dpldp[k+jth] = 0.5*(th2-th1);
    }
}

return ALL_OKAY;
}

/*********************************************************************/
SDDAS_SHORT nswps2QiCDF_lev_vars(QiCDFVariable *levv, QiCDFVariable *levv_delta,
                                  PEACE_SWEEP **swppp, int nswps){
/**********************************************************************/
int iswp, jlev, k,  nlev, i;
float *levvp, *levv_dldp, lev1, lev2;
long num_atts = 13;
long num_datts = 6;

/* array attributes for cef are Value_type,  UNITS, SI_conversion,
*        Bin_location, Scaling; COULD ADD Frame & Component_desc 
*        Sizes and Value_type added by qie routines */
/* number, data_type, and num_elems set by nswps2QiCDF_default_attr */


char *att_names[13] = {"TENSOR_RANK","UNITS", "SI_CONVERSION", "LABLAXIS",
       "FRAME", "REPRESENTATION", "COMPONENT", "FIELDNAM", 
       "DELTA_MINUS", "DELTA_PLUS", "TENSOR_FRAME", "PARAMETER_TYPE","VAR_TYPE"};
char *att_data[13]  = {"1","dummy units", "dummy SI conversion", "dummy labl",
    "to be overwritten", "rtp", "r", "dummy field",
    "level_delta","level_delta", "pea", "Support_Data","metadata"};

char *datt_names[6] = {"UNITS", "SI_CONVERSION", "LABLAXIS", "FIELDNAM","PARAMETER_TYPE","VAR_TYPE"};
char *datt_data[6]  = {"dummy units", "dummy SI","dummy labl", "dummy field","Support_Data","metadata"};


levv->num_v_attrs= num_atts;
levv_delta->num_v_attrs= num_datts;
for (i=0; i<num_atts; i++) {
  nswps2QiCDF_default_attr( &(levv->attribute[i]),i );
  levv->attribute[i].name = QiNewStr(att_names[i]);
  levv->attribute[i].data = QiNewStr(att_data[i]);
}
levv->attribute[0].data_type=CDF_INT4;  /* rank */
PQiChangeVattr("TENSOR_RANK",att_data[0],levv);


for (i=0; i<num_datts; i++) {
  nswps2QiCDF_default_attr( &(levv_delta->attribute[i]),i );
  levv_delta->attribute[i].name = QiNewStr(datt_names[i]);
  levv_delta->attribute[i].data = QiNewStr(datt_data[i]);
}

PQiChangeVattr("SI_CONVERSION",
         BIN_UNITS_SI_conv_STRING(swppp[0]->bin_units),levv);
PQiChangeVattr("SI_CONVERSION",
         BIN_UNITS_SI_conv_STRING(swppp[0]->bin_units),levv_delta);

switch(swppp[0]->bin_units){
    case BIN_UNITS_EV :
      PQiChangeVattr("LABLAXIS","Energy",levv);
      PQiChangeVattr("UNITS","eV",levv);
      PQiChangeVattr("FRAME","scalar>na",levv);
      PQiChangeVattr("FIELDNAM","Peace Energy",levv); 
      PQiChangeVattr("LABLAXIS","Energy",levv_delta);
      PQiChangeVattr("UNITS","eV",levv_delta);
      PQiChangeVattr("FIELDNAM","Peace Energy",levv_delta); 
      break;
    case BIN_UNITS_KM_S :
      PQiChangeVattr("LABLAXIS","Speed",levv);
      PQiChangeVattr("UNITS","km/s",levv);
      PQiChangeVattr("FRAME","component>pea_rtp",levv);
      PQiChangeVattr("FIELDNAM","Peace Speed",levv); 
      PQiChangeVattr("LABLAXIS","Speed",levv_delta);
      PQiChangeVattr("UNITS","km/s",levv_delta);
      PQiChangeVattr("FIELDNAM","Peace Speed",levv_delta); 
      break;
    default :
      PQiChangeVattr("LABLAXIS","Level (unknown)",levv);
      PQiChangeVattr("FRAME","scalar>na",levv);
      PQiChangeVattr("FIELDNAM","Peace Level (unknown)",levv); 
      PQiChangeVattr("LABLAXIS","Level (unknown)",levv_delta);
      PQiChangeVattr("FIELDNAM","Peace Level (unknown)",levv_delta); 
      break;
}

nlev = swppp[0]->n_lev;

levv->data        = (float *)malloc(nswps * nlev * sizeof(float));
levvp             = (float *)levv->data;
levv_delta->data  = (float *)malloc(nswps * nlev * sizeof(float));
levv_dldp         = (float *)levv_delta->data;

/* data is data[nswp][nlev] so data[i][j = data[i*nlev + j] */

for (iswp=0; iswp<nswps; iswp++){
    k = iswp*nlev;       /* indx of 1st elem in data for swpp[iswp] */
    for (jlev = 0; jlev < nlev; jlev++){
       lev1              = swppp[iswp]->elo[jlev];
       lev2              = swppp[iswp]->ehi[jlev];
       
       levvp[k+jlev]     = 0.5*(lev1 + lev2);
       levv_dldp[k+jlev] = 0.5*(lev2-lev1);
    }
}

return ALL_OKAY;
}


/*********************************************************************/
SDDAS_SHORT nswps2QiCDF_phisc_var(QiCDFVariable *phisc, 
                                  PEACE_SWEEP **swppp, int nswps){
/**********************************************************************/

int i;
float *phiscdp;
long num_atts = 7;  /* fillval added may2002; Depend_end_0 added Feb2003 */
                     /* SC_id and FIELDNAM added oct 2003  */

/* array attributes for cef are Value_type,  FILLVAL, UNITS, SI_conversion,
*        Depend_1, Depend_end_1, Depend_2, Depend_end_2, 
*        Bin_description, Bin_description_A        
*        Sizes and Value_type added by qie routines */
/* number, data_type,  set by nswps2QiCDF_default_attr */

char *att_names[7] = {"UNITS", "SI_CONVERSION","DEPEND_0","SC_ID", "FIELDNAM","PARAMETER_TYPE","VAR_TYPE"};
char *att_data[7]  = {"V", "1>V", "timetags","dummy sc", "s/c Potential","Support_Data","metadata"};

phisc->num_v_attrs= num_atts;
for (i=0; i<num_atts; i++) {
  nswps2QiCDF_default_attr( &(phisc->attribute[i]),i );
  phisc->attribute[i].name = QiNewStr(att_names[i]);
  phisc->attribute[i].data = QiNewStr(att_data[i]);
}

PQiChangeVattr("SC_ID", swppp[0]->lineage->mission,phisc);

phisc->data     = (float *)malloc(nswps  * sizeof(float));

phiscdp = (float *)phisc->data;

/* fill data array. NB psd is a 2-D array, but in right order so copy as 1-D*/
for (i=0; i<nswps; i++){
    phiscdp[i] = swppp[i]->phi_sc;
}

return ALL_OKAY;
}



/********************************************************************/
long   PQiChangeVattr(char *attr_name, 
                     char *new_entry,
                     QiCDFVariable *var)
/*******************************************************************/
/* sjs reinstates this here from QiChangeVattr in older qie. For some
*  reason this was removed in move to cef2 version */            
{
   long n;
   long n_attr =-1;
   float *float_data;
   int *int_data;

   /* find named attr */
   for (n=0; n<var->num_v_attrs; n++){
     if( strcmp(attr_name, (var->attribute + n)->name) == 0 ){
       n_attr = n;
       break;
     }
   }

   if(n_attr == -1){
     n_attr = var->num_v_attrs;
     var->num_v_attrs++;       /* note we initialised with SPARE attr ptrs */
     (var->attribute +n_attr)->number = n_attr;
     (var->attribute +n_attr)->name = QiNewStr(attr_name);
     (var->attribute +n_attr)->data_type = CDF_CHAR; 
   }

   /* replace existing entry: ONLY char type handled at present */

   switch ( (var->attribute +n_attr)->data_type){
    case CDF_EPOCH: case CDF_REAL8: case CDF_DOUBLE:

      break;
      
    case CDF_REAL4: case CDF_FLOAT:
      if( (var->attribute +n_attr)->data != NULL ) 
           free((var->attribute +n_attr)->data);
      float_data = (float *)malloc(sizeof(float));
      *float_data= (float)atof(new_entry);        
      (var->attribute +n_attr)->data= (void *)float_data;  
      (var->attribute +n_attr)->num_elems = 1;

      break;
    
    case CDF_INT4: case CDF_UINT4:
      if( (var->attribute +n_attr)->data != NULL ) 
           free((var->attribute +n_attr)->data);
      int_data = (int *)malloc(sizeof(int));
      *int_data= (int)atoi(new_entry);  
      (var->attribute +n_attr)->data= (void *)int_data;  
      (var->attribute +n_attr)->num_elems = 1;

      break;
      
    case CDF_INT2: case CDF_UINT2:

      break;
    
    case CDF_INT1: case CDF_UINT1: case CDF_BYTE: 

      break;

    case CDF_CHAR: case CDF_UCHAR:
      if( (var->attribute +n_attr)->data != NULL ) 
           free((var->attribute +n_attr)->data);
      (var->attribute +n_attr)->data = QiNewStr(new_entry);  
      (var->attribute +n_attr)->num_elems = 1;
      break;

   }
 

   return QMW_OK;

} /* end QiChangeVattr */


/* copied from qie_qsas.h , and made dummy call */
int QiCEFmsg (int mode, const char *txt0,
              const char *txt1,
              const char *txt2){

  return 0;
}




#ifdef __cplusplus
}
#endif
