/* qjas.cc ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s.j.schwartz@qmw.ac.uk)
*  Queen Mary, University of London
*  26 November 2000
*  last up-date 16 July 2001
*  March 2002 - implemented zero as missing value
*
*  Utility routines for generic used, including:
*
*  Setting strings, idfs lineages, etc.
*  Setting conversion tables, getting energy bin limits, ...
*  Opeining and positioning idfs file
*  Manipulating time into format needed by and returned from idfs routines
*
*/


#include <stdio.h>

#include "qjas.h"

 /*
 * Constructor
 *
 * Creates child widgets of the main QjasBox widget
 */

QjasBox::QjasBox( QWidget *parent, const char *name )
    : QVBox( parent, name )
{
    setMargin( 5 );
    setSpacing( 5 );

    QString str;

    QHBox *row1 = new QHBox( this );
    row1->setSpacing( 5 );

    // Create a LineageBox which has comboboxes for the various lineage components
    lineage = new LineageBox( row1 );
 
/*    QVBox *timecol = new QVBox (row1);
       timecol->setSpacing(5);
       QLabel *timecol_label = new QLabel("Start and End Times", timecol);
       timecol_label->setAlignment(AlignLeft);
    starttime = new TimeBox(1, Horizontal, timecol);

    QHBox *copy_time_box  = new QHBox(timecol);
    QLabel *copy_time_label = new QLabel("Copy Time   ", copy_time_box);
       copy_time_label->setAlignment( AlignRight | AlignVCenter);
    QPushButton *copy_time = new QPushButton ("|\nV",copy_time_box);
    new QLabel("     ",copy_time_box);
    connect (copy_time, SIGNAL (clicked() ),
             this , SLOT (slotCopyTime() ) );

    endtime = new TimeBox(1, Horizontal, timecol);
*/
// ***
    QVBox *timecol = new QVBox (row1);
    timecol->setSpacing(10);
    QLabel *plottypelabel = new QLabel("Plot type",timecol);
       plottypelabel->setAlignment(Qt::AlignCenter);
    plottype  = new QComboBox(FALSE,timecol);
       plottype->insertItem("QJAS Energy-Angle-Time Spectrogram",-1);
       plottype->insertItem("Wheel Sequence (Xwin or File)",-1);
       plottype->insertItem("Wheel Sequence + cuts (Xwin or File)",-1);
    QLabel *plottypespace = new QLabel(" ",timecol);

    startendbox = new TimeBox(timecol,"Start/End Times");
    startendbox->setGeometry( QRect( 11, 10, 500, 150 ) );
// ***
    QHBox *scale_opts = new QHBox (this);
    scale_opts->setSpacing(5);
    QVBox *scale_labels       = new QVBox (scale_opts);
    	                        scale_labels->setSpacing(5);
    QVBox *scale_units        = new QVBox (scale_opts);
    	                        scale_units->setSpacing(5);
    QVBox *scale_linlog       = new QVBox (scale_opts);
    	                        scale_linlog->setSpacing(5);
    QVBox *scale_auto         = new QVBox (scale_opts);
    	                        scale_auto->setSpacing(5);
    QVBox *scale_minmax_label = new QVBox (scale_opts);
    	                        scale_minmax_label->setSpacing(5);
    QVBox *scale_min          = new QVBox (scale_opts);
    	                        scale_min->setSpacing(5);
    QVBox *scale_max          = new QVBox (scale_opts);
     	                        scale_max->setSpacing(5);

// Energy/velocity options
    new QLabel ("Energy/Velocity Options", scale_labels);
    energy_units = new QComboBox(FALSE,scale_units);
       for (int ibu = 0; ibu < N_BIN_UNITS; ibu++){
          energy_units->insertItem (BIN_UNITS_STRING((BIN_UNITS)ibu), -1);
       }
    energy_bins = new QComboBox(FALSE,scale_linlog);
       energy_bins->insertItem("Instrument Bins",-1);
       energy_bins->insertItem("Custom Bins",-1);
   energy_sc_pot = new QComboBox (FALSE,scale_auto);
       energy_sc_pot->insertItem("No s/c pot. correction",-1);
       energy_sc_pot->insertItem("Do s/c pot. correction",-1);
       energy_sc_pot->insertItem("Const s/c pot.",-1);
       QToolTip::add(energy_sc_pot,
                  "Uses EFW for Cluster; otherwise only const used");      
   new QLabel ("pot gap val, secs",scale_minmax_label);
   sc_pot_val      = new QLineEdit ("0.0",scale_min);
      QToolTip::add(sc_pot_val, "S/c pot (eV) for gap or constant value");
   sc_pot_gap_secs = new QLineEdit ("60.0",scale_max);
      QToolTip::add(sc_pot_gap_secs, "Interval > this considered gap in EFW");
// Custom bins options
    new QLabel ("   Custom bins: Number", scale_labels);
    nebins = new QLineEdit("10",scale_units);
    energy_linlog = new QComboBox(FALSE,scale_linlog);
       energy_linlog->insertItem("Linear",-1);
       energy_linlog->insertItem("Log",-1);
       energy_linlog->setCurrentItem(1);
   energy_scale = new QComboBox(FALSE,scale_auto);
       energy_scale->insertItem("Instrument Range",-1);
       energy_scale->insertItem("User Range ->",-1);
     new QLabel ("min, max", scale_minmax_label);
    emin = new QLineEdit ("10.00", scale_min);
    emax = new QLineEdit ("1000.00", scale_max);

// PSD Options
    new QLabel ("Phase Space Options", scale_labels);
    psd_units = new QComboBox(FALSE,scale_units);
       for (int ipsu = 0; ipsu < N_PSD_UNITS; ipsu++){
          psd_units->insertItem (PSD_UNITS_STRING((PSD_UNITS)ipsu), -1);
       }
    psd_linlog = new QComboBox(FALSE,scale_linlog);
       psd_linlog->insertItem("Linear",-1);    /* same order as PGSpectro.h */
       psd_linlog->insertItem("Log",-1);
       psd_linlog->setCurrentItem(1);
    psd_scale = new QComboBox(FALSE,scale_auto);
       psd_scale->insertItem("Autoscale",-1); /* same order as PGSpectro.h */
       psd_scale->insertItem("Userscale->",-1);
    new QLabel ("min, max", scale_minmax_label);
    psdmin = new QLineEdit ("1.00000", scale_min);
//    psdmin->setFixedWidth(70);
    psdmin->setMinimumWidth(70);
    psdmax = new QLineEdit ("10.0000", scale_max);
//    psdmax->setFixedWidth(70);
    psdmax->setMinimumWidth(70);
    
// Plot Options
    QHBox *plot_opts = new QHBox (this);
    plot_opts->setSpacing(5);
    new QLabel ("Plot Options", plot_opts);
    plot_col = new QComboBox(FALSE,plot_opts);
       plot_col->insertItem("Rainbow",-1);   /* same order as PGSpectro.h */
       plot_col->insertItem("Gray White Low",-1);
       plot_col->insertItem("Gray White High", -1);
       plot_col->insertItem("Inv Red-Blue",-1);  
       plot_col->insertItem("Red-Blue",-1);
       plot_col->insertItem("Earth", -1);
    new QLabel ("t-fill gaps < (s)", plot_opts);
    plot_tgap = new QLineEdit("0.0",plot_opts); plot_tgap->setFixedWidth(45);
    new QLabel ("Xwin Number", plot_opts);
    plot_win_edit = new QLineEdit ("3", plot_opts);
    plot_win_edit->setMaxLength(3); plot_win_edit->setFixedWidth(45);
    plot_dev = new QComboBox(FALSE,plot_opts);
       plot_dev->insertItem("Landscape GIF File ->",-1);
       plot_dev->insertItem("Portrait GIF File ->",-1);
       plot_dev->insertItem("Landscape PS File ->",-1);
       plot_dev->insertItem("Portrait PS File ->",-1);
    new QLabel ("Plot File Stem", plot_opts);
    plot_file_edit = new QLineEdit ("qjas_plot", plot_opts);
    QPushButton *plotfileselect = new QPushButton("Browse...",plot_opts);
    connect(plotfileselect, SIGNAL(clicked()), this, SLOT (slotPlotfile()) );

// Export Options
    QHBox *exp_opts_box = new QHBox (this);
    exp_opts_box->setSpacing(5);
    new QLabel ("Export to File Options", exp_opts_box);
    exp_opt = new QComboBox(FALSE,exp_opts_box);
       exp_opt->insertItem("Ascii Cluster Exchange 2",-1);
       exp_opt->insertItem("Ascii Cluster Exchange 1",-1);
       exp_opt->insertItem("Ascii Tabular",-1);
       exp_opt->insertItem("CDF", -1);
    new QLabel ("Export File Stem", exp_opts_box);
    exp_file_edit = new QLineEdit ("qjas_data", exp_opts_box);
    QPushButton *expfileselect = new QPushButton("Browse...",exp_opts_box);
    connect(expfileselect, SIGNAL(clicked()), this, SLOT (slotExpfile()) );


// Action Buttons
    QHButtonGroup *actions = new QHButtonGroup (this);
    connect (actions, SIGNAL(clicked(int)), this, SLOT (slotPlot(int)));
 
    new QPushButton( "Plot Xwin",actions );
    new QPushButton( "Print to File",actions );
    
    QPushButton *exprtbut = new QPushButton( "Export", actions);
    QPushButton *help = new QPushButton( "Help", actions);
    QPushButton *save = new QPushButton("Save...",actions);
    QPushButton *restore = new QPushButton("Restore...",actions);
    connect(exprtbut,  SIGNAL(clicked()),  this, SLOT ( slotExp()) );

    connect (help,     SIGNAL(clicked()) , this, SLOT (slotHelp()) );
    connect (save,     SIGNAL(clicked()),  this, SLOT (slotSave()));
    connect (restore,  SIGNAL(clicked()),  this, SLOT (slotRestore()));
    quit = new QPushButton( "Quit",actions);

// create help window 
    helpbwin = new Helpbwin(); 

}



/*
*     Constructor for LineageBox 
*/

LineageBox::LineageBox( QWidget *parent, const char *name )
    : QVBox( parent, name )
{
    this->setSpacing(5);
    new QLabel("IDFS Lineage", this);
    proj = new  QComboBox (FALSE, this);
    proj->insertStrList (project_strs);
    miss = new  QComboBox (FALSE, this);
    for (int isc = 0; isc < N_SC; isc++){
          miss->insertItem (SC_NAME((SC)isc), -1);
    }
    exp  = new  QComboBox (FALSE, this);
    exp->insertStrList (exp_strs);
    inst = new  QComboBox (FALSE, this);
    inst->insertStrList (inst_strs);
    virins = new  QComboBox (FALSE, this);
    virins->insertStrList (virins_strs);
    pachoice = new QComboBox (FALSE, this);
       pachoice->insertItem("no pa binning", -1);
       pachoice->insertItem("pa: angles", -1);
       pachoice->insertItem("pa: cosines", -1);    
    npa = new QLineEdit("13", this);
      QToolTip::add(npa, "Number of pitch angle bins for rebinning");
    gatherchoice = new QComboBox (FALSE, this);
       gatherchoice->insertItem("Gather n sweeps",-1);
       gatherchoice->insertItem("Gather n spins",-1);
       gatherchoice->insertItem("Gather n.n secs",-1);
    gathern = new QLineEdit("1",this);   
}

void LineageBox::grab_lineage(IDFS_LINEAGE *lngp){

strcpy(lngp->project,proj->currentText().latin1());
strcpy(lngp->mission,miss->currentText().latin1());
strcpy(lngp->experiment,exp->currentText().latin1());
strcpy(lngp->instrument,inst->currentText().latin1());
strcpy(lngp->virtual_inst,virins->currentText().latin1());
strcpy(lngp->exten,"");
if (DIAG_PRINT_3D)print_idfs_lineage(lngp);

}

/*
*     Constructors for TimeBox
*/

/*TimeBox::TimeBox (int strips, Orientation orientation,
                          QWidget *parent, const char *name )
     : QGroupBox (strips, orientation, parent, name)
{

QHBox *row_time = new QHBox (this);
row_time->setSpacing(30);
new QLabel("     ",row_time);  // space
time_lbl   = new QLabel ("yyyy doy hh:mm:ss.sss", row_time);
year = new QLineEdit ("2001", row_time);
year->setMaxLength(4);year->setFixedWidth(60);
doy = new QLineEdit ("090", row_time);
doy->setMaxLength(3);doy->setFixedWidth(45);
hr = new QLineEdit ("18", row_time);
hr->setMaxLength(2);hr->setFixedWidth(30);
min = new QLineEdit ("8", row_time);
min->setMaxLength(2);min->setFixedWidth(30);
sec = new QLineEdit ("12.345", row_time);
sec->setMaxLength(6);sec->setFixedWidth(75);
new QLabel("     ",row_time);  // space

}
*/

TimeBox:: TimeBox(QWidget *parent, const char *name)
    :  QGroupBox(parent,name)
{

    byearLabel = new QLabel( "Year", this);
    byearLabel->setGeometry( QRect( 10, 40, 40, 20 ) );

    StartTimeLabel = new QLabel( "Start", this );
    StartTimeLabel->setGeometry( QRect( 10, 20, 64, 20 ) );

    bdoyLabel = new QLabel( "Doy", this );
    bdoyLabel->setGeometry( QRect( 70, 40, 40, 20 ) );

    bYearspinBox = new QSpinBox( this );
    bYearspinBox->setGeometry( QRect( 10, 60, 60, 23 ) );
    bYearspinBox->setWrapping( TRUE );
    bYearspinBox->setMaxValue( 2100 );
    bYearspinBox->setMinValue( 1900 );
    bYearspinBox->setValue( 2001 );

    textLabel1_2 = new QLabel( "Date", this );
    textLabel1_2->setGeometry( QRect( 10, 85, 40, 20 ) );

    btimeEdit = new QTimeEdit( this );
    btimeEdit->setGeometry( QRect( 125, 60, 84, 23 ) );
    btimeEdit->setAutoAdvance( TRUE );
    this->setSHour(18);

    bDoyspinBox = new QSpinBox( this );
    bDoyspinBox->setGeometry( QRect( 70, 60, 55, 23 ) );
    bDoyspinBox->setWrapping( TRUE );
    bDoyspinBox->setMaxValue( 366 );
    bDoyspinBox->setMinValue( 1 );
    bDoyspinBox->setValue(90);

    StartdateEdit = new QDateEdit( this );
    StartdateEdit->setEnabled( TRUE );
    StartdateEdit->setGeometry( QRect( 10, 105, 100, 23 ) );
    StartdateEdit->setDate( QDate( 2001, 3, 31 ) );
    StartdateEdit->setOrder( QDateEdit::YMD);
    StartdateEdit->setAutoAdvance( TRUE );
    
    btimeLabel = new QLabel( "Time", this );
    btimeLabel->setGeometry( QRect( 125, 40, 64, 20 ) );

    EnddateEdit = new QDateEdit( this );
    EnddateEdit->setGeometry( QRect( 289, 105, 100, 23 ) );
    EnddateEdit->setDate( QDate( 2001, 3, 31 ) );
    EnddateEdit->setOrder(QDateEdit::YMD);
    EnddateEdit->setAutoAdvance( TRUE );

    eYearspinBox = new QSpinBox( this );
    eYearspinBox->setGeometry( QRect( 290, 60, 60, 23 ) );
    eYearspinBox->setWrapping( TRUE );
    eYearspinBox->setMaxValue( 2100 );
    eYearspinBox->setMinValue( 1900 );
    eYearspinBox->setValue( 2001 );

    textLabel1_2_2 = new QLabel( "Date", this );
    textLabel1_2_2->setGeometry( QRect( 290, 85, 40, 20 ) );

    EndTimetextLabel = new QLabel(  "End", this );
    EndTimetextLabel->setGeometry( QRect( 290, 20, 65, 20 ) );

    eyearLabel = new QLabel( "Year" , this);
    eyearLabel->setGeometry( QRect( 290, 40, 40, 20 ) );

    etimeLabel = new QLabel( "Time", this );
    etimeLabel->setGeometry( QRect( 405, 40, 64, 20 ) );

    edoyLabel = new QLabel(  "Doy", this );
    edoyLabel->setGeometry( QRect( 350, 40, 40, 20 ) );

    eDoyspinBox = new QSpinBox( this );
    eDoyspinBox->setGeometry( QRect( 350, 60, 55, 23 ) );
    eDoyspinBox->setWrapping( TRUE );
    eDoyspinBox->setMaxValue( 366 );
    eDoyspinBox->setMinValue( 1 );
    eDoyspinBox->setValue(90);

    etimeEdit = new QTimeEdit( this );
    etimeEdit->setGeometry( QRect( 405, 60, 84, 23 ) );
    etimeEdit->setAutoAdvance( TRUE );
    this->setEHour(18);
    this->setEMinute(1);

    CopyTimepushButton = new QPushButton(  ">" , this);
    CopyTimepushButton->setGeometry( QRect( 230, 60, 40, 20 ) );
    resize( QSize(522, 531).expandedTo(minimumSizeHint()) );
    CopyTimepushButton->setAutoDefault( FALSE );

    connect( CopyTimepushButton, SIGNAL( clicked() ), this, SLOT( CopyTime() ) );
    connect( StartdateEdit, SIGNAL( valueChanged(const QDate&) ), this, SLOT( StartDateChange() ) );
    connect( EnddateEdit, SIGNAL( valueChanged(const QDate&) ), this, SLOT( EndDateChange() ) );
    connect( bYearspinBox, SIGNAL( valueChanged(int) ), this, SLOT( StartYearChange() ) );
    connect( eYearspinBox, SIGNAL( valueChanged(int) ), this, SLOT( EndYearChange() ) );
    connect( bDoyspinBox, SIGNAL( valueChanged(int) ), this, SLOT( StartDoyChange() ) );
    connect( eDoyspinBox, SIGNAL( valueChanged(int) ), this, SLOT( EndDoyChange() ) );

    setTabOrder( bYearspinBox, bDoyspinBox );
    setTabOrder( bDoyspinBox, StartdateEdit );
    setTabOrder( btimeEdit, CopyTimepushButton );
    setTabOrder( StartdateEdit, btimeEdit );
    setTabOrder( CopyTimepushButton, eYearspinBox );
    setTabOrder( eYearspinBox, eDoyspinBox );
    setTabOrder( eDoyspinBox, EnddateEdit );
    setTabOrder( EnddateEdit, etimeEdit);
}    


void TimeBox::CopyTime()
{
    int bYear = bYearspinBox->value();
    int bDoy = bDoyspinBox->value();
    int eDoy = bDoy;
    int eYear = bYear;
    QTime bTime = btimeEdit->time();
    QTime eTime = bTime;
    eYearspinBox->setValue(eYear);
    eDoyspinBox->setValue(eDoy);
    etimeEdit->setTime(eTime);   
}


void TimeBox::StartDateChange()
{
    int *doy = new int;
    QDate *sdate = new QDate;
    *sdate = StartdateEdit->date();
    
    QanDateToYearDoy(sdate->year(), sdate->month(), sdate->day(), doy);
    if((sdate->year() >= 1950) && (sdate->year()<=2100))
    { 
    bYearspinBox->setValue(sdate->year());
    bDoyspinBox->setValue(*doy);}        
}

void TimeBox::EndDateChange()
{
    int *doy = new int;
    QDate *edate = new QDate;
    *edate = EnddateEdit->date();
    
    QanDateToYearDoy(edate->year(), edate->month(), edate->day(), doy);
    
    if((edate->year() >= 1950) && (edate->year()<=2100))
    {eYearspinBox->setValue(edate->year());
    eDoyspinBox->setValue(*doy);}  
}

void TimeBox::StartYearChange()
{
   int *month = new int;
   int *day = new int;
   int year, doy;
   
   year =  bYearspinBox->value();
   doy  =  bDoyspinBox->value();  
   
   QanYearDoyToDate(year, doy, month, day);   
   QDate date(year, *month, *day);   
   StartdateEdit->setDate(date);     
}

void TimeBox::EndYearChange()
{
   int *month = new int;
   int *day = new int;
   int year, doy;
   
   year = eYearspinBox->value();
   doy = eDoyspinBox->value();
   
   QanYearDoyToDate(year, doy, month, day);   
   QDate date(year, *month, *day);   
   EnddateEdit->setDate(date);     
}

void TimeBox::StartDoyChange()
{
   int *month = new int;
   int *day = new int;
   int year, doy;
   
   year = bYearspinBox->value();
   doy = bDoyspinBox->value();
   
   QanYearDoyToDate(year, doy, month, day);   
   QDate date(year, *month, *day);   
   StartdateEdit->setDate(date);     
}

void TimeBox::EndDoyChange()
{
   int *month = new int;
   int *day = new int;
   int year, doy;
   
   year = eYearspinBox->value();
   doy = eDoyspinBox->value();
   
   QanYearDoyToDate(year, doy, month, day);
   QDate date(year, *month, *day);   
   EnddateEdit->setDate(date);     
}


void TimeBox::setSHour(int hour)
{
int min, sec;
QTime time = btimeEdit->time();
min = time.minute();
sec = time.second();
time.setHMS(hour,min,sec);
btimeEdit->setTime(time);
}

void TimeBox::setSMinute(int minute)
{
int hour, sec;
QTime time = btimeEdit->time();
hour = time.hour();
sec  = time.second();
time.setHMS(hour,minute,sec);
btimeEdit->setTime(time);
}

void TimeBox::setSSecond(int second)
{
int hour, min;
QTime time = btimeEdit->time();
hour = time.hour();
min  = time.minute();
time.setHMS(hour,min,second);
btimeEdit->setTime(time);
}


void TimeBox::setEHour(int hour)
{
int min, sec;
QTime time = etimeEdit->time();
min = time.minute();
sec = time.second();
time.setHMS(hour,min,sec);
etimeEdit->setTime(time);
}

void TimeBox::setEMinute(int minute)
{
int hour, sec;
QTime time = etimeEdit->time();
hour = time.hour();
sec  = time.second();
time.setHMS(hour,minute,sec);
etimeEdit->setTime(time);
}

void TimeBox::setESecond(int second)
{
int hour, min;
QTime time = etimeEdit->time();
hour = time.hour();
min  = time.minute();
time.setHMS(hour,min,second);
etimeEdit->setTime(time);
}

void TimeBox::GetStartTimeFromTimeBox(IDFS_TIME *idftp){

/*int year, doy, hr, min, isec;
long nano;
float secs;

year = this->year->text().toInt();
doy  = this->doy->text().toInt();
hr   = this->hr->text().toInt();
min  = this->min->text().toInt();
secs = this->sec->text().toFloat();

isec = (int)secs;
nano = (long)(1000000000*(secs-isec));

to_idfs_time(year, doy, hr, min, isec, nano, idftp);
if (DIAG_PRINT_3D)print_idfs_time(idftp);
*/

int year, doy, hr, min, isec;
long nano;
QTime time;

year = bYearspinBox->value();
doy  = bDoyspinBox->value();

time = btimeEdit->time();
hr   = time.hour();
min  = time.minute();
isec = time.second();
nano = 0;

to_idfs_time(year, doy, hr, min, isec, nano, idftp);
if (DIAG_PRINT_3D)print_idfs_time(idftp);

}

void TimeBox::GetEndTimeFromTimeBox(IDFS_TIME *idftp){

int year, doy, hr, min, isec;
long nano;
QTime time;

year = eYearspinBox->value();
doy  = eDoyspinBox->value();

time = etimeEdit->time();
hr   = time.hour();
min  = time.minute();
isec = time.second();
nano = 0;

to_idfs_time(year, doy, hr, min, isec, nano, idftp);
if (DIAG_PRINT_3D)print_idfs_time(idftp);
}


/* Constructor and destructor for help window */

Helpbwin::Helpbwin(QWidget *parent, const char *name) 
   : QVBox(parent,name){

resize(600,600);
setCaption("qjas help");

QHBox *buttons = new QHBox(this);
QPushButton *backbutton, *forwardbutton, *topbutton;

backbutton = new QPushButton( "<- Back", buttons, 0 );
forwardbutton = new QPushButton ("Forward ->",buttons, 0);
topbutton = new QPushButton ("Top",buttons,0);

QTextBrowser *helpbrowser = new QTextBrowser(this);

connect( backbutton, SIGNAL( clicked() ), helpbrowser, SLOT( backward() ) );
connect( forwardbutton, SIGNAL( clicked() ), helpbrowser, SLOT( forward() ) );
connect( topbutton, SIGNAL( clicked() ), helpbrowser, SLOT( home() ) );

QString homepath = QString(getenv("QPEACE_HOME")) + "/doc/";

helpbrowser->mimeSourceFactory()->setFilePath(homepath);
helpbrowser->setSource("qjas.html");

}

Helpbwin::~Helpbwin(void){

hide();
}


/* void QjasBox::slotCopyTime(void){
endtime->year->setText( starttime->year->text() );
endtime->doy->setText( starttime->doy->text() );
endtime->hr->setText( starttime->hr->text() );
endtime->min->setText( starttime->min->text() );
endtime->sec->setText( starttime->sec->text() );
}
*/
void QjasBox::slotPlot(int pltbutton){

SDDAS_SHORT ret_val;
int is_pad, nswp_pa, ngath, nsc, nlev, nsen, i;
PEACE_SWEEP **swppp_pa, **gath_swppp;
PEACE_SWEEP *binh;
PEACE_PITCH_ANGLE_OPTS single_paopts;
float *lobins, *hibins;
float emin_val, emax_val;
PEACE_GATHER_GAPS gapopts;
PEACE_GATHER_CHOICE gatherch;
float gathersecs;
int gather_swp_spins;
QString s; 
DEV_CHOICE device;

SWEEP_WHEEL_OPTIONS swopts;  /* for wheel anims */
PEACE_SWEEP *cut_swps[2];
SWEEP_CUT_OPTIONS scutopt;
QPGLine_colour linecols[2]={QPGBLACK,QPGBLACK};
QPGLinestyle linestys[2]={QPGSOLID,QPGDASHED};
float linethick[2]={2.0,2.0};
float thetas[2];

lineage->grab_lineage(&lng1);

startendbox->GetStartTimeFromTimeBox(&idft1);
startendbox->GetEndTimeFromTimeBox(&idft2);

bin_units = (BIN_UNITS)energy_units->currentItem();
psdu = (PSD_UNITS)psd_units->currentItem();

PGSpectro_set_defaults(&pgspectro);

pgspectro.color_table = (PGSpectro_col)plot_col->currentItem();
pgspectro.zlinlog     = (PGSpectro_linlog)psd_linlog->currentItem();
Yes_or_No zautoscale  = (Yes_or_No)psd_scale->currentItem();
pgspectro.zscalemin        = psdmin->text().toFloat();
pgspectro.zscalemax        = psdmax->text().toFloat();
pgspectro.skip_missing     = YES;
pgspectro.zmissing         = QP_DEFAULT_MISSING; /* missing value reset below*/
pgspectro.xgapsize         = plot_tgap->text().toFloat();
if (pgspectro.xgapsize > 0.0) pgspectro.xgapfill = YES;

QString pgfile, pgfileexten;
QString pgdev, pgdevstr;

T_OR_F animation;
T_OR_F cuts;

pgfile=plot_file_edit->text(); // get file name; if xwin, reset below

switch((PLOT_CHOICE)plottype->currentItem() ){
   case WHEEL_ANIM:
      animation = QPTRUE; cuts = QPFALSE; break;
   case WHEEL_CUTS_ANIM:
      animation = QPTRUE; cuts = QPTRUE; break;
   case QJAS: default:
      animation = QPFALSE; cuts = QPFALSE; break;
}   

if (pltbutton==0) device= XWIN;
else if (pltbutton==1){
   device = (DEV_CHOICE)plot_dev->currentItem();
   /* if (animation==QPTRUE){
      if (device != XWIN) device = PGIF;
   } */
} else return;   /* other actions connected to their own slots */


switch(device){
   case LGIF:
      pgdev.setLatin1("/GIF",-1);
      if (animation==QPTRUE) pgfileexten.setLatin1("#.gif",-1);
      else pgfileexten.setLatin1(".gif", -1);
      break;
    case PGIF:
      pgdev.setLatin1("/VGIF",-1);
      if (animation==QPTRUE) pgfileexten.setLatin1("#.gif",-1);
      else pgfileexten.setLatin1(".gif", -1);
      break;
   case LPS:
      pgdev.setLatin1("/CPS",-1);
      if (animation==QPTRUE) pgfileexten.setLatin1(".ps",-1);
      else  pgfileexten.setLatin1(".eps", -1);
      break;
   case PPS:
      pgdev.setLatin1("/VCPS",-1);
      if (animation==QPTRUE) pgfileexten.setLatin1(".ps",-1);
      else pgfileexten.setLatin1(".eps", -1);
      break;
   case XWIN:  default:
      bool ok;
      pgfile = plot_win_edit->text();
      int ixwin = pgfile.toInt( &ok, 10);  // see if text in pgfile ok xwin
      if (!ok || ixwin < 1 ||ixwin > 255){ // if not open default xwin
          pgfile.setLatin1(PLOTAZ_DEFAULT_XWIN_STR, -1); 
      }
      pgdev.setLatin1("/Xserv", -1);
      pgfileexten.setLatin1("");
      break;
}

char pg_dev_cstr[500];
pgdevstr = pgfile.append(pgfileexten);
pgdevstr.append(pgdev);
strcpy(pg_dev_cstr, pgdevstr.latin1());
// swoptions.pg_device =  pg_dev_cstr;

if (cpgopen(pg_dev_cstr) <=0){
 printf("Failed to open pgplot device %s successfully\n", pg_dev_cstr);
 printf("If file check write permissions; if Xwindow, check pgxwin server\n");
 return;       // early exit, before getting sweeps and malloc memory
}
cpgshls(0,0,1,0);
cpgshls(1,0,0,0);

is_pad = strcmp("PAD",lng1.instrument);
if (is_pad==0) is_pad=1; else is_pad = 0;
int pach = lineage->pachoice->currentItem();
if (pach == NO_BINNING){
   paopts.do_pa = QPFALSE;
} else {
   paopts.do_pa = QPTRUE;
   if (pach == PA_ANGLES) paopts.mu_bins = QPFALSE;
       else paopts.mu_bins = QPTRUE;
   paopts.n_pabins = lineage->npa->text().toInt();
}
swppp =  NULL; // for safety

if (strncmp(lng1.mission,"C1234",5)){nsc=1;}else{nsc=4;}/* single s/c plot? */

cpgsubp(1,nsc);
int isc;
for (isc = 0; isc < nsc; isc++){
    cpgpage();
    PGSpectro_domain_set_defaults(&(pgspectro.domain));
    pgspectro.wedge.disp = 2.5;  /* leave room for Flow 0 180 label */
    if (nsc == 4) {
       pgspectro.char_size = 1.4;
       strcpy(lng1.mission, SC_NAME((SC_E)isc));
    }

    int phisc_ch = energy_sc_pot->currentItem();
    switch ( phisc_ch ){
       case NO_SC_POT_CORR:
          ret_val = GetNPeaceSweeps(&lng1,&idft1,&idft2,bin_units,psdu,
               paopts.do_pa, &nswp, &swppp);
          break;
       case DO_SC_POT_CORR :  /* not const s/c pot */
          ret_val = GetNPotCorrectedPeaceSweeps(
	        &lng1,&idft1,&idft2,bin_units,psdu,paopts.do_pa, 
		QPFALSE, sc_pot_val->text().toFloat(),
		sc_pot_gap_secs->text().toFloat(),
		&nswp, &swppp);
	  break;
       case CONST_SC_POT :
          ret_val = GetNPotCorrectedPeaceSweeps(
	        &lng1,&idft1,&idft2,bin_units,psdu,paopts.do_pa, 
		QPTRUE, sc_pot_val->text().toFloat(),
		sc_pot_gap_secs->text().toFloat(),
		&nswp, &swppp);
	  break;
       default:
          printf("Unknown SC Potential Choice");
	  ret_val = PEACE3D_ERROR;
	  break;
    }
    if (ret_val != ALL_OKAY){
       printf("QJAS failed initial Get Sweeps returning %d.",ret_val);
       printf(" Try another source/time?\n");
       break;
    }        

  if(DIAG_QJAS_SWEEPS){
     printf("QJAS finished initial GetNSweeps (incl pot correction\nFirst sweep:\n");
     print_sweep_structure(swppp[0]);
  }

/* set up bins for rebinning, but don't do it yet */
/* Note pa-rebinning requires congruent sweeps: same levs and sensors */
/* Note rebin_n_sweeps does NOT copy pitch angles ! */
/* So one approach is:
*
*  Rebin each sweep (one by one) into pitch angles - at initial angle resol.
*  Then Gather/combine onto final bin spec:
*
*/

    if (paopts.do_pa == QPTRUE){   /* 1st rebin each sweep to pa; old bins */
       single_paopts = paopts;
       single_paopts.n_pabins = swppp[0]->n_sen;
       single_paopts.mu_bins  = QPFALSE;    /* keep as close as poss to orig*/
       
       ret_val = Rebin_n_sweeps_to_pa(swppp, nswp, 1, &single_paopts,
                 &swppp_pa,&nswp_pa);
       free_n_sweeps(nswp,swppp); free(swppp); swppp=NULL; 
                                          /*  free orig ptr*/ 
       if (ret_val != ALL_OKAY){
          printf("Qjas failed to rebin each sweep in pa, ret = %d\n",ret_val);
	  break;	  
       }
       nswp = nswp_pa;
       swppp=swppp_pa;   swppp_pa = NULL;
    }

    binh = create_sweep_full_copy(swppp[0]); /* use existing unless:*/
    if (energy_bins->currentItem() == 1){ // custom bins requested
	 free(binh->elo);
	 free(binh->ehi);
	 nlev = binh->n_lev = nebins->text().toInt(); // set up ebins
	 if ( energy_scale->currentItem() == 1){            // user scale
	     emin_val = emin->text().toFloat();
	     emax_val = emax->text().toFloat();
	 } else {  /* be careful as could have negative bins */
	     i=0;
	     while (i<swppp[0]->n_lev) {
	       emin_val=swppp[0]->elo[i++];
	       if (emin_val > 0.0) break;
	     }  
	     if (emin_val < 0) emin_val = SAFE_EMIN;
	     emax_val = swppp[0]->ehi[swppp[0]->n_lev -1];
	     if (emax_val < 0) emax_val = SAFE_EMAX;
	 }
	 binh->elo = (float *)malloc(nlev*sizeof(float));
	 binh->ehi = (float *)malloc(nlev*sizeof(float));
	 set_equal_bins_linlog(binh->elo, binh->ehi, nlev, 
	            emin_val, emax_val, 
		    (PGSpectro_linlog)energy_linlog->currentItem());
    }
    if (paopts.do_pa == QPTRUE){
	free(binh->thlo); free(binh->thhi);
	nsen = binh->n_sen = paopts.n_pabins;
	lobins = binh->thlo = (float *)malloc(nsen*sizeof(float));
	hibins = binh->thhi = (float *)malloc(nsen*sizeof(float));
	if (paopts.mu_bins == QPTRUE){
	   set_equal_bins( lobins, hibins, nsen, 1.0, -1.0);
	   for (i=0; i<nsen; i++){
	      lobins[i] = RAD2DEG*acos( lobins[i]);
	      hibins[i] = RAD2DEG*acos( hibins[i]);
	   }
	   lobins[0] = 0.0;  /* force to elim rounding error */
	   hibins[nsen -1] = 180.0;
	} else {
	   set_equal_bins (lobins, hibins, nsen, 0.0, 180.0);
	}
    }
    /* psd and pa arrays of binh not needed; gather will try to make
       a full sweep copy, which may not be same sizes; create_sweep_full_copy
       traps psd and pa == NULL, so safe. */
    if(binh->psd !=NULL){free(binh->psd); binh->psd=NULL;}
    if(binh->pa !=NULL){free(binh->pa); binh->pa=NULL;}

    sweep_spin_gap_defaults (&gapopts);  /* default swp, spin gap */
    if (is_pad) gapopts.sweep_gap = gapopts.spin_gap;
    
    gatherch= (PEACE_GATHER_CHOICE)lineage->gatherchoice->currentItem();
    gathersecs = lineage->gathern->text().toFloat();
    gather_swp_spins = lineage->gathern->text().toInt();
    
    ret_val = gather_sweeps(swppp, nswp, binh, gatherch, gapopts, gathersecs,
       gather_swp_spins, &gath_swppp, &ngath);
    if(DIAG_QJAS_SWEEPS){
	     printf("QJAS finished gather sweeps\nFirst sweep:\n");
	     print_sweep_structure(gath_swppp[0]);
    }

    Set_sweep_wheel_options_default(&swopts);
    swopts.rmin = binh->elo[0];
    swopts.rmax = binh->ehi[binh->n_lev -1];
    free_sweep_arrays(binh); free(binh);
    if (ret_val != ALL_OKAY){
       printf("gather sweeps failed returning %d\n",ret_val);
       free_n_sweeps(nswp,swppp); free(swppp);   swppp=NULL;
       break ;
    }
    free_n_sweeps(nswp,swppp); free(swppp); 
    swppp = gath_swppp; nswp = ngath; gath_swppp = NULL;  /* shuffle */  
    
    if(zautoscale == YES && isc == 0){
      sweeps_autoscaler(swppp, nswp, 
                    &(pgspectro.zscalemin), &(pgspectro.zscalemax)); 
    }
    pgspectro.zmissing = swppp[0]->missing_value; 

    if (animation == QPTRUE){
      swopts.color_table  = pgspectro.color_table;
      swopts.zlinlog      = pgspectro.zlinlog;
      swopts.rlinlog      = (PGSpectro_linlog)energy_linlog->currentItem();

      swopts.zautoscale   = zautoscale;
      swopts.rautoscale   = NO;
      swopts.zmin         = pgspectro.zscalemin;
      swopts.zmax         = pgspectro.zscalemax;
      swopts.skip_missing = YES;
      cong_swps = NULL;
      
//      cong_swps = (PEACE_SWEEP **)malloc(nswp*sizeof (PEACE_SWEEP *));
      if (cuts == QPTRUE) {
          /* cpgsubp(2,1); cpgpage(); */
	  swopts.text_scale=0.6;
 	  thetas[0]=0; thetas[1]=90;
	  Set_Sweep_cut_options_default(&scutopt);
	  scutopt.yautoscale = NO;
	  scutopt.xautoscale = NO;
	  scutopt.ylinlog = swopts.zlinlog;
	  scutopt.xlinlog = swopts.rlinlog;
	  scutopt.nswps=2;
	  scutopt.colours = linecols;
	  scutopt.linestyles = linestys;
	  scutopt.linethicknesses = linethick;
	  scutopt.ymin = swopts.zmin;
	  scutopt.ymax = swopts.zmax;
	  scutopt.xmin = swopts.rmin;
	  scutopt.xmax = swopts.rmax;
	  swopts.xndcmin = 0.1; swopts.xndcmax = 0.4;
	  swopts.yndcmin = 0.1; swopts.yndcmax = 0.8;
	  scutopt.xndcmin = 0.5; scutopt.xndcmax = 0.95;
	  scutopt.yndcmin = 0.25; scutopt.yndcmax = 0.65;
      }
      for (int i=0; i<nswp; i++){
//	   cong_swps[i] = rebin_sweep(swppp[0], swppp[i]);
//           pg_ret_val = PlotTwoSweeps(cong_swps[i],cong_swps[i],&swopts);
            pg_ret_val = PlotTwoSweeps(swppp[i],swppp[i],&swopts);
	   if (cuts == QPTRUE){
	     cut_swps[0] = cut_swps[1] = swppp[i];
	     pg_ret_val = Sweep_n_cuts_plot (cut_swps,2,thetas,&scutopt);
	   }
           cpgpage();      
       }    
       emin->setText(s.setNum(swppp[0]->elo[0]));  /* set ebins in gui */
       emax->setText(s.setNum(swppp[0]->ehi[ swppp[0]->n_lev -1 ]));
//       free_n_sweeps(nswp,cong_swps);  free(cong_swps);    cong_swps=NULL;

    } else {
      pg_ret_val=Qjas_plot(swppp, nswp, &pgspectro, idft1, idft2);
      emin->setText(s.setNum(swppp[0]->elo[0]));  /* set ebins in gui */
      emax->setText(s.setNum(swppp[0]->ehi[ swppp[0]->n_lev -1 ]));
      if (pg_ret_val == PGSPECTRO_QJAS_INCONGRUENT){
	cong_swps = (PEACE_SWEEP **)malloc(nswp * sizeof (PEACE_SWEEP *));
	for (int i=0; i<nswp; i++){
	   cong_swps[i] = rebin_sweep(swppp[0], swppp[i]);
	}
	pgspectro.zmissing = cong_swps[0]->missing_value; 
	pg_ret_val=Qjas_plot(cong_swps, nswp, &pgspectro, idft1, idft2);
	emin->setText(s.setNum(cong_swps[0]->elo[0])); /* set ebins in gui */
	emax->setText(s.setNum(cong_swps[0]->ehi[cong_swps[0]->n_lev -1]));
	free_n_sweeps(nswp,cong_swps);  free(cong_swps);    cong_swps=NULL;
      }
    }       
    psdmin->setText(s.setNum(pgspectro.zscalemin));
    psdmax->setText(s.setNum(pgspectro.zscalemax));
    free_n_sweeps(nswp,swppp);       free(swppp);        swppp=NULL;
}
cpgclos();   /* make sure pgplot dev is properly closed */
}  /* end slotPlot */

/*********************************************** also add code to rebin energies
*****************************************************************************/

void QjasBox::slotExpfile(void){

QString expfile = QFileDialog::getSaveFileName( 
  this->exp_file_edit->text() , QString::null   , this,0,
   "Specify Export File Stem");

if(!expfile.isEmpty())this->exp_file_edit->setText(expfile);
}

void QjasBox::slotPlotfile(void){

QString plotfile = QFileDialog::getSaveFileName( 
  this->plot_file_edit->text() , QString::null   , this,0,
   "Specify Plot File Stem");

if(!plotfile.isEmpty())this->plot_file_edit->setText(plotfile);
}


void QjasBox::slotExp(void){

SDDAS_SHORT ret_val;
QiCDFContents *QiCDF;
QiOptions *QiOpts;
long check;
QString expfile;

int is_pad, nswp_pa, ngath, nlev, nsen, i;
PEACE_SWEEP **swppp_pa, **gath_swppp;
PEACE_SWEEP *binh;
PEACE_PITCH_ANGLE_OPTS single_paopts;
float *lobins, *hibins;
float emin_val, emax_val;
PEACE_GATHER_GAPS gapopts;
PEACE_GATHER_CHOICE gatherch;
float gathersecs;
int gather_swp_spins;


QiCDF = NULL;
QiOpts = NULL;

lineage->grab_lineage(&lng1);

startendbox->GetStartTimeFromTimeBox(&idft1);
startendbox->GetEndTimeFromTimeBox(&idft2);

bin_units = (BIN_UNITS)energy_units->currentItem();
psdu = (PSD_UNITS)psd_units->currentItem();

is_pad = strcmp("PAD",lng1.instrument);
if (is_pad==0) is_pad=1; else is_pad = 0;
int pach = lineage->pachoice->currentItem();
if (pach == NO_BINNING){
   paopts.do_pa = QPFALSE;
} else {
   paopts.do_pa = QPTRUE;
   if (pach == PA_ANGLES) paopts.mu_bins = QPFALSE;
       else paopts.mu_bins = QPTRUE;
   paopts.n_pabins = lineage->npa->text().toInt();
}
swppp = NULL;

if (strncmp(lng1.mission,"C1234",5)==0){ /* multi-s/c not exportable */
  printf("Multi-spacecraft export not supported\n");
  ret_val = PEACE_SWEEPS_EXPORT_ERROR;
} else {  /* looks ok; keep going */
//
// ************************************** get sweeps & bin in pa if requested
//
   int phisc_ch = energy_sc_pot->currentItem();
   switch ( phisc_ch ){
      case NO_SC_POT_CORR:
	 ret_val = GetNPeaceSweeps(&lng1,&idft1,&idft2,bin_units,psdu,
              paopts.do_pa, &nswp, &swppp);
	 break;
      case DO_SC_POT_CORR :  /* not const s/c pot */
	 ret_val = GetNPotCorrectedPeaceSweeps(
	       &lng1,&idft1,&idft2,bin_units,psdu,paopts.do_pa, 
	       QPFALSE, sc_pot_val->text().toFloat(),
	       sc_pot_gap_secs->text().toFloat(),
	       &nswp, &swppp);
	 break;
      case CONST_SC_POT :
	 ret_val = GetNPotCorrectedPeaceSweeps(
	       &lng1,&idft1,&idft2,bin_units,psdu,paopts.do_pa, 
	       QPTRUE, sc_pot_val->text().toFloat(),
	       sc_pot_gap_secs->text().toFloat(),
	       &nswp, &swppp);
	 break;
      default:
	 printf("Unknown SC Potential Choice");
	 ret_val = PEACE3D_ERROR;
	 break;
   }
   if (ret_val != ALL_OKAY){
      printf("QJAS failed initial Get Sweeps returning %d.",ret_val);
      printf(" Try another source/time?\n");
   }        
       
   /* set up bins for rebinning, but don't do it yet */
   /* Note pa-rebinning requires congruent sweeps: same levs and sensors */
   /* Note rebin_n_sweeps does NOT copy pitch angles ! */
   /* So one approach is:
   *
   *  Rebin each sweep (one by one) into pitch angles - at initial angle resol.
   *  Then Gather/combine onto final bin spec:
   *
   */

    if (paopts.do_pa == QPTRUE){   /* 1st rebin each sweep to pa; old bins */
       single_paopts = paopts;
       single_paopts.n_pabins = swppp[0]->n_sen;
       single_paopts.mu_bins  = QPFALSE;    /* keep as close as poss to orig*/
       
       ret_val = Rebin_n_sweeps_to_pa(swppp, nswp, 1, &single_paopts,
                 &swppp_pa,&nswp_pa);
       if (ret_val != ALL_OKAY){
          printf("Qjas failed to rebin each sweep in pa, ret = %d\n",ret_val);
       }
       nswp = nswp_pa;
       free_n_sweeps(nswp,swppp); free(swppp);   /* shuffle to orig ptr*/ 
       swppp=swppp_pa;   swppp_pa = NULL;
    }

    binh = create_sweep_full_copy(swppp[0]); /* use existing unless:*/
    if (energy_bins->currentItem() == 1){ // custom bins requested
	 free(binh->elo);
	 free(binh->ehi);
	 nlev = binh->n_lev = nebins->text().toInt(); // set up ebins
	 if ( energy_scale->currentItem() == 1){            // user scale
	     emin_val = emin->text().toFloat();
	     emax_val = emax->text().toFloat();
	 } else {  /* be careful as could have negative bins */
	     i=0;
	     while (i<swppp[0]->n_lev) {
	       emin_val=swppp[0]->elo[i++];
	       if (emin_val > 0.0) break;
	     }  
	     if (emin_val < 0) emin_val = SAFE_EMIN;
	     emax_val = swppp[0]->ehi[swppp[0]->n_lev -1];
	     if (emax_val < 0) emax_val = SAFE_EMAX;
	 }
	 binh->elo = (float *)malloc(nlev*sizeof(float));
	 binh->ehi = (float *)malloc(nlev*sizeof(float));
	 set_equal_bins_linlog(binh->elo, binh->ehi, nlev, 
	            emin_val, emax_val, 
		    (PGSpectro_linlog)energy_linlog->currentItem());
    }
    if (paopts.do_pa == QPTRUE){
	free(binh->thlo); free(binh->thhi);
	nsen = binh->n_sen = paopts.n_pabins;
	lobins = binh->thlo = (float *)malloc(nsen*sizeof(float));
	hibins = binh->thhi = (float *)malloc(nsen*sizeof(float));
	if (paopts.mu_bins == QPTRUE){
	   set_equal_bins( lobins, hibins, nsen, 1.0, -1.0);
	   for (i=0; i<nsen; i++){
	      lobins[i] = RAD2DEG*acos( lobins[i]);
	      hibins[i] = RAD2DEG*acos( hibins[i]);
	   }
	   lobins[0] = 0.0;  /* force to elim rounding error */
	   hibins[nsen -1] = 180.0;
	} else {
	   set_equal_bins (lobins, hibins, nsen, 0.0, 180.0);
	}
    }

    sweep_spin_gap_defaults (&gapopts);  /* default swp, spin gap */
    if (is_pad) gapopts.sweep_gap = gapopts.spin_gap;
    
    gatherch= (PEACE_GATHER_CHOICE)lineage->gatherchoice->currentItem();
    gathersecs = lineage->gathern->text().toFloat();
    gather_swp_spins = lineage->gathern->text().toInt();
    
    ret_val = gather_sweeps(swppp, nswp, binh, gatherch, gapopts, gathersecs,
       gather_swp_spins, &gath_swppp, &ngath);
    if (ret_val != ALL_OKAY){
       printf("gather sweeps failed returning %d\n",ret_val);
       free_n_sweeps(nswp,swppp); free(swppp);   swppp=NULL;
    } else {
       free_n_sweeps(nswp,swppp); free(swppp);
       swppp = gath_swppp; nswp = ngath; gath_swppp = NULL;  /* shuffle */  
    }
}
//
// *************************************** fill CDFContents Obj
//
if(ret_val == ALL_OKAY){ 
    ret_val = nsweeps2QiCDFContents( swppp, nswp, &QiCDF);

     //
     //*******************************************  set opts and export       	  
     //
   if (ret_val == ALL_OKAY){

       QiOpts = QiMakeOptionsObj();
       QiOpts->header = ATTACHED;     QiOpts->rec_numbering = NUM_OFF;
       QiOpts->priority = REPLACE;    QiOpts->object_type = TS;
       QiOpts->attr_delim = ',';      QiOpts->data_delim = ',';
       QiOpts->time_sep_attr = 'T';   QiOpts->time_sep_data = 'T';
       QiOpts->fp_display = stdout;

       // set up output file path/name - only exten left till later
       expfile=(exp_file_edit->text()).simplifyWhiteSpace(); 
                    // get file name - stem only, strip all white space; 
       char *temp_str = STR_NULL;
       temp_str = (char *)malloc ((strlen(expfile.latin1()) + 1)*sizeof(char));
       strcpy(temp_str, expfile.latin1());
 
       int ilastslash = expfile.findRev("/",-1);
       if (ilastslash < 0){   // no slash found
           QiCDF->io_f_path = STR_NULL;
	   QiCDF->io_f_name = QiNewStr(temp_str);
       } else {
           QiCDF->io_f_name = QiNewStr(temp_str+ilastslash +1);
	   temp_str[ilastslash+1] = '\0';
	   QiCDF->io_f_path = QiNewStr(temp_str);
       }
       if (temp_str != STR_NULL) free(temp_str);
       
       printf("io_f_path = |%s|, io_f_name = |%s|\n",
           QiCDF->io_f_path, QiCDF->io_f_name);

       switch(exp_opt->currentItem()){
	  case 3:      //cdf
	     check = QiWriteCSDSgenCDF(QiCDF,QiOpts);
	     if (check != QMW_OK){
        	     printf("QiWriteCSDSgenCDF returned %ld\n", check);
		     printf("%s",QiErrStr(check));
	     }
	     break;
	    case 2:      // qft
	     QiOpts->f_type    = TABULAR;
	     check = QiWriteCSDSgenFlat(QiCDF,QiOpts);
	     if (check != QMW_OK){
        	     printf("QiWriteCSDSgenFlat returned %ld\n", check);
		     printf("%s",QiErrStr(check));
	     }
	     break;
	  case 1:       // cef
	     QiOpts->f_type    = EXCHANGE; // formerly DELIMITED;
	     QiOpts->attr_delim = ',';
             QiOpts->data_delim = ',';
             QiOpts->rec_end = '$';
             QiOpts->row_end = '\n';
	     check = QiWriteCSDSgenFlat(QiCDF,QiOpts);
	     if (check != QMW_OK){
        	     printf("QiWriteCSDSgenFlat returned %ld\n", check);
		     printf("%s",QiErrStr(check));
	     }
	     break;
	  case 0: default:     // cef2
	     QiOpts->f_type    = CAA; 
	     QiOpts->attr_delim = ',';
             QiOpts->data_delim = ',';
             QiOpts->rec_end = '$';
             QiOpts->row_end = '\n';
	     check = QiWriteCSDSgenFlat(QiCDF,QiOpts);
	     if (check != QMW_OK){
        	     printf("QiWriteCSDSgenFlat returned %ld\n", check);
		     printf("%s",QiErrStr(check));
	     }
	     break;
       }  // end switch on exp_opt->currentItem

   } else {   // problem putting sweeps into CDFContents Obj
     printf("nsweeps2QiCDFContents returned %d\n", ret_val);
   }

   QiFreeOptionsObj (QiOpts);    // should be safe
   QiFreeCDFContentsObj( QiCDF); // ditto 
   free_n_sweeps(nswp,swppp);     
   free(swppp);      
   swppp=NULL;

} else {
   printf("GetNPeaceSweeps returned error %d. Try another source/time?\n",
       ret_val);
}

}


void QjasBox::slotHelp(void){
if ( QApplication::desktop()->width() < 600
            || QApplication::desktop()->height() < 600 ){
   helpbwin->showMaximized();
} else {
   helpbwin->show();  
}
}


void QjasBox::slotSave(void){
FILE *out;
QString savefile = QFileDialog::getSaveFileName( );
out = fopen(savefile.latin1(),"w");
if (out == NULL){
   printf("Error opening %s for save\n",savefile.latin1());
   return;
}   
fprintf(out, "%s = %d\n", savestrings[SPROJECT],     lineage->proj->currentItem());
fprintf(out, "%s = %d\n", savestrings[SMISSION],     lineage->miss->currentItem());
fprintf(out, "%s = %d\n", savestrings[SEXPERIMENT],  lineage->exp->currentItem());
fprintf(out, "%s = %d\n", savestrings[SINSTRUMENT],  lineage->inst->currentItem());
fprintf(out, "%s = %d\n", savestrings[SVIRTUAL],     lineage->virins->currentItem());
fprintf(out, "%s = %d\n", savestrings[SPA_OPTS],     lineage->pachoice->currentItem());
fprintf(out, "%s = %d\n", savestrings[SN_PA],        lineage->npa->text().toInt());
fprintf(out, "%s = %d\n", savestrings[SGATHER_OPTS], lineage->gatherchoice->currentItem());
fprintf(out, "%s = %g\n", savestrings[SN_GATHER],    lineage->gathern->text().toFloat());
fprintf(out, "%s = %d\n", savestrings[SSTART_YR],    startendbox->bYearspinBox->value());
fprintf(out, "%s = %d\n", savestrings[SSTART_DOY],   startendbox->bDoyspinBox->value());
fprintf(out, "%s = %d\n", savestrings[SSTART_HR],    startendbox->btimeEdit->time().hour());
fprintf(out, "%s = %d\n", savestrings[SSTART_MIN],   startendbox->btimeEdit->time().minute());
fprintf(out, "%s = %d\n", savestrings[SSTART_SEC],   startendbox->btimeEdit->time().second());
fprintf(out, "%s = %d\n", savestrings[SEND_YR],      startendbox->eYearspinBox->value());
fprintf(out, "%s = %d\n", savestrings[SEND_DOY],     startendbox->eDoyspinBox->value());
fprintf(out, "%s = %d\n", savestrings[SEND_HR],      startendbox->etimeEdit->time().hour());
fprintf(out, "%s = %d\n", savestrings[SEND_MIN],     startendbox->etimeEdit->time().minute());
fprintf(out, "%s = %d\n", savestrings[SEND_SEC],     startendbox->etimeEdit->time().second());
fprintf(out, "%s = %d\n", savestrings[SPLOT_TYPE],   plottype->currentItem());
fprintf(out, "%s = %d\n", savestrings[SENERGY_UNITS],energy_units->currentItem());
fprintf(out, "%s = %d\n", savestrings[SENERGY_BINS], energy_bins->currentItem());
fprintf(out, "%s = %d\n", savestrings[SSC_POT_OPTS], energy_sc_pot->currentItem());
fprintf(out, "%s = %g\n", savestrings[SSC_POT_VAL],  sc_pot_val->text().toFloat());
fprintf(out, "%s = %g\n", savestrings[SSC_POT_SECS], sc_pot_gap_secs->text().toFloat());
fprintf(out, "%s = %g\n", savestrings[SCUSTOM_BIN_NO],     nebins->text().toFloat());
fprintf(out, "%s = %d\n", savestrings[SCUSTOM_BIN_LINLOG], energy_linlog->currentItem());
fprintf(out, "%s = %d\n", savestrings[SCUSTOM_BIN_RANGE],  energy_scale->currentItem());
fprintf(out, "%s = %g\n", savestrings[SCUSTOM_BIN_MIN],    emin->text().toFloat());
fprintf(out, "%s = %g\n", savestrings[SCUSTOM_BIN_MAX],    emax->text().toFloat());
fprintf(out, "%s = %d\n", savestrings[SPSD_UNITS],    psd_units->currentItem());
fprintf(out, "%s = %d\n", savestrings[SPSD_LINLOG],   psd_linlog->currentItem());
fprintf(out, "%s = %d\n", savestrings[SPSD_SCALE],    psd_scale->currentItem());
fprintf(out, "%s = %g\n", savestrings[SPSD_MIN],      psdmin->text().toFloat());
fprintf(out, "%s = %g\n", savestrings[SPSD_MAX],      psdmax->text().toFloat());
fprintf(out, "%s = %d\n", savestrings[SPLOT_COL],     plot_col->currentItem());
fprintf(out, "%s = %d\n", savestrings[SPLOT_DEV],     plot_dev->currentItem());
fprintf(out, "%s = %d\n", savestrings[SXWIN_NO],      plot_win_edit->text().toInt());
fprintf(out, "%s = %s\n", savestrings[SPLOT_STEM],    plot_file_edit->text().latin1());
fprintf(out, "%s = %s\n", savestrings[SPLOT_TGAP],    plot_tgap->text().latin1());
fprintf(out, "%s = %d\n", savestrings[SEXPORT_FORMAT],exp_opt->currentItem());
fprintf(out, "%s = %s\n", savestrings[SEXPORT_STEM],  exp_file_edit->text().latin1());

fclose(out);

return;

}

void QjasBox::slotRestore(void){

char inputline[MAXLENGTH_SAVE_LINE];

int i;

FILE *in;
QString infile = QFileDialog::getOpenFileName();
in = fopen(infile.latin1(),"r");
if (in == NULL){
   printf("Error opening %s for restore\n",infile.latin1());
   return;
}

QString qin, qparam, qvalue;

while (fgets (inputline, MAXLENGTH_SAVE_LINE, in) != NULL){
    
    qin    = inputline;
    qparam = (qin.section("=",0,0)).stripWhiteSpace();
    qvalue = (qin.section("=",-1,-1)).stripWhiteSpace();

    //printf("qparam = |%s| and qvalue = |%s|\n",qparam.latin1(),qvalue.latin1());

    for (i=0; i<SDUMMYSTRING; i++){
        if(qparam ==savestrings[i])break;
    }
    if (i< SDUMMYSTRING){
       switch(i){
       
         case SPROJECT:     lineage->proj->setCurrentItem(qvalue.toInt());      break;
         case SMISSION:     lineage->miss->setCurrentItem(qvalue.toInt());      break;
         case SEXPERIMENT:  lineage->exp->setCurrentItem(qvalue.toInt());       break;
         case SINSTRUMENT:  lineage->inst->setCurrentItem(qvalue.toInt());      break;
         case SVIRTUAL:     lineage->virins->setCurrentItem(qvalue.toInt());    break;
         case SPA_OPTS:     lineage->pachoice->setCurrentItem(qvalue.toInt());  break;
         case SN_PA:        lineage->npa->setText(qvalue);                      break;
         case SGATHER_OPTS: lineage->gatherchoice->setCurrentItem(qvalue.toInt());break;
         case SN_GATHER:    lineage->gathern->setText(qvalue);                  break;
         case SSTART_YR:    startendbox->bYearspinBox->setValue(atoi(qvalue));  break;
         case SSTART_DOY:   startendbox->bDoyspinBox->setValue(atoi(qvalue));   break;
         case SSTART_HR:    startendbox->setSHour(atoi(qvalue));                break;
         case SSTART_MIN:   startendbox->setSMinute(atoi(qvalue));              break;
         case SSTART_SEC:   startendbox->setSSecond(atoi(qvalue));              break;
         case SEND_YR:      startendbox->eYearspinBox->setValue(atoi(qvalue));  break;
         case SEND_DOY:     startendbox->eDoyspinBox->setValue(atoi(qvalue));   break;
         case SEND_HR:      startendbox->setEHour(atoi(qvalue));                break;
         case SEND_MIN:     startendbox->setEMinute(atoi(qvalue));              break;
         case SEND_SEC:     startendbox->setESecond(atoi(qvalue));              break;
	 case SPLOT_TYPE:   plottype->setCurrentItem(qvalue.toInt());           break;
         case SENERGY_UNITS:energy_units->setCurrentItem(qvalue.toInt());       break;
         case SENERGY_BINS: energy_bins->setCurrentItem(qvalue.toInt());        break;
         case SSC_POT_OPTS: energy_sc_pot->setCurrentItem(qvalue.toInt());      break;
         case SSC_POT_VAL:  sc_pot_val->setText(qvalue);                        break;
         case SSC_POT_SECS: sc_pot_gap_secs->setText(qvalue);                   break;
         case SCUSTOM_BIN_NO:     nebins->setText(qvalue);                      break;
         case SCUSTOM_BIN_LINLOG: energy_linlog->setCurrentItem(qvalue.toInt());break;
         case SCUSTOM_BIN_RANGE:  energy_scale->setCurrentItem(qvalue.toInt()); break;
         case SCUSTOM_BIN_MIN:    emin->setText(qvalue);                        break;
         case SCUSTOM_BIN_MAX:    emax->setText(qvalue);                        break;
         case SPSD_UNITS:   psd_units->setCurrentItem(qvalue.toInt());          break;
         case SPSD_LINLOG:  psd_linlog->setCurrentItem(qvalue.toInt());         break;
         case SPSD_SCALE:   psd_scale->setCurrentItem(qvalue.toInt());          break;
         case SPSD_MIN:     psdmin->setText(qvalue);                            break;
         case SPSD_MAX:     psdmax->setText(qvalue);                            break;
         case SPLOT_COL:    plot_col->setCurrentItem(qvalue.toInt());           break;
         case SPLOT_DEV:    plot_dev->setCurrentItem(qvalue.toInt());           break;
         case SXWIN_NO:     plot_win_edit->setText(qvalue);                     break;
         case SPLOT_STEM:   plot_file_edit->setText(qvalue);                    break;
         case SPLOT_TGAP:   plot_tgap->setText(qvalue);                         break;
         case SEXPORT_FORMAT: exp_opt->setCurrentItem(qvalue.toInt());          break;
         case SEXPORT_STEM:   exp_file_edit->setText(qvalue);                   break;	 
	 
	 default: break;
       }
    }

}

}
